using System;
using System .Collections .Generic;
using System .Drawing;
using System .Drawing .Drawing2D;

using MoveGraphLibrary;

namespace Test_MoveGraphLibrary
{
    public enum ShowValue { AsNumber, AsPercent };

    // class DoughnutRing will be the part of DoughnutSet
    // and DoughnutSet will be derived from GraphicalObject
    public class DoughnutRing
    {
        Point ptCenter;
        int nInRadius;
        int nOutRadius;
        double [] fVal;
        Color [] clrs;
        Color [] clrsText;
        Color clrBorder;
        Font fontTexts;
        bool bShowTexts;
        ShowValue showvalue;
        Rotation dirFlow;
        int nStartAngle_Deg;
        bool bShowBorders;
        double [] fStart_Deg;
        double [] fSweep_Deg;

        int nMinInRadius = 20;   // min size is set to avoid disappearence while resizing
        int nMinWidth = 10;   // min size is set to avoid disappearence while resizing
        Rectangle rcFrame;      // calculated while executing Draw()

        // -------------------------------------------------        DoughnutRing
        public DoughnutRing (Point ptC, int nInside, int nOutside, double [] values, Font fnt)
        {
            // FIRST: upgrade input values to what they have to be
            nInside = Math .Max (Math .Abs (nInside), nMinInRadius);
            nOutside = Math .Max (Math .Abs (nOutside), nInside + nMinWidth);
            if (values == null || values .Length < 2)
            {
                values = new double [] { 1, 2, 3, 4, 5 };
            }
            int nSector = values .Length;

            // NOW define the new object
            ptCenter = ptC;
            nInRadius = nInside;
            nOutRadius = nOutside;
            fVal = new double [nSector];
            clrs = new Color [nSector];
            clrsText = new Color [nSector];
            fStart_Deg = new double [nSector];
            fSweep_Deg = new double [nSector];

            for (int i = 0; i < nSector; i++)
            {
                if (values [i] <= 0)
                {
                    values [i] = 0.5;
                }
                fVal [i] = values [i];
            }
            for (int i = 0; i < nSector; i++)
            {
                clrs [i] = Auxi_Colours .ColorPredefined (i);
                clrsText [i] = Color .Black;
            }

            clrBorder = Color .Black; ;
            fontTexts = fnt;
            bShowTexts = true;
            showvalue = ShowValue .AsNumber;
            dirFlow = Rotation .Clockwise;
            nStartAngle_Deg = 0;
            bShowBorders = true;

            StartAndSweep (ref fStart_Deg, ref fSweep_Deg);
        }
        // =================================================        StartAndSweep
        private void StartAndSweep (ref double [] fStart, ref double [] fSweep)
        {
            int nSectors = fVal .Length;
            double fTotal = 0;
            for (int i = 0; i < nSectors; i++)
            {
                fTotal += Math .Abs (fVal [i]);
            }
            // for calculations fStart[] and fSweep[] are used in common way: positive means counterclock
            for (int i = 0; i < nSectors; i++)
            {
                fSweep_Deg [i] = 360.0 * (Math .Abs (fVal [i]) / fTotal);
                if (dirFlow == Rotation .Clockwise)
                {
                    fSweep_Deg [i] *= -1;
                }
            }
            fStart_Deg [0] = nStartAngle_Deg;
            for (int i = 1; i < nSectors; i++)
            {
                fStart_Deg [i] = fStart_Deg [i - 1] + fSweep_Deg [i - 1];
            }
        }
        // -------------------------------------------------        ChangeRotation
        public void ChangeRotation ()
        {
            dirFlow = (dirFlow == Rotation .Clockwise) ? Rotation .Counterclock : Rotation .Clockwise;
        }
        // -------------------------------------------------        Draw
        public void Draw (Graphics grfx)
        {
            Rectangle rcIn = new Rectangle (ptCenter .X - nInRadius, ptCenter .Y - nInRadius, 2 * nInRadius, 2 * nInRadius);
            Rectangle rcOut = new Rectangle (ptCenter .X - nOutRadius, ptCenter .Y - nOutRadius, 2 * nOutRadius, 2 * nOutRadius);
            int nRadMiddle = (nInRadius + nOutRadius) / 2;
            Rectangle rcMiddle = new Rectangle (ptCenter .X - nRadMiddle, ptCenter .Y - nRadMiddle,
                                                2 * nRadMiddle, 2 * nRadMiddle);
            StartAndSweep (ref fStart_Deg, ref fSweep_Deg);
            int nSectors = fVal .Length;
            Pen penBorder = new Pen (clrBorder);
            Point pt0, pt1;
            float fStart, fSweep;
            // for Drawing fStart[] and fSweep[] are used in Microsoft way, which means changing of sign
            double fSum = 0.0;
            for (int i = 0; i < fVal .Length; i++)
            {
                fSum += fVal [i];
            }
            GraphicsPath path = new GraphicsPath ();
            for (int i = 0; i < nSectors; i++)
            {
                SolidBrush brush = new SolidBrush (clrs [i]);
                fStart = (float) (-fStart_Deg [i]);
                fSweep = (float) (-fSweep_Deg [i]);

                path .AddArc (rcIn, fStart, fSweep);
                path .AddArc (rcOut, fStart + fSweep, -fSweep);
                grfx .FillPath (brush, path);
                path .Reset ();

                if (bShowBorders)
                {
                    grfx .DrawArc (penBorder, rcIn, fStart, fSweep);
                    grfx .DrawArc (penBorder, rcOut, fStart, fSweep);
                    pt0 = Auxi_Geometry .EllipseBorder (rcIn, -fStart, 1);
                    pt1 = Auxi_Geometry .EllipseBorder (rcOut, -fStart, 1);
                    grfx .DrawLine (penBorder, pt0, pt1);
                }
                if (bShowTexts)
                {
                    Point ptMiddle = Auxi_Geometry .EllipseBorder (rcMiddle, -fStart - fSweep / 2, 1);
                    string str;
                    switch (showvalue)
                    {
                        case ShowValue .AsNumber:
                            str = fVal [i] .ToString ();
                            Auxi_Drawing .RotatedText (grfx, str, fontTexts, 0, clrsText [i], ptMiddle, TextBasicPoint .M);
                            break;
                        case ShowValue .AsPercent:
                            str = Convert .ToInt32 ((fVal [i] * 100 / fSum)) .ToString () + "%";
                            Auxi_Drawing .RotatedText (grfx, str, fontTexts, 0, clrsText [i], ptMiddle, TextBasicPoint .M);
                            break;
                    }
                }
            }
            rcFrame = rcOut;
        }
        // -------------------------------------------------        InsideMainArea
        public bool InsideMainArea (Point pt)
        {
            int nRad = Convert .ToInt32 (Auxi_Geometry .Distance (pt, ptCenter));
            return (nInRadius <= nRad && nRad <= nOutRadius);
        }
        // -------------------------------------------------        Center
        public Point Center
        {
            get { return (ptCenter); }
            set { ptCenter = value; }
        }
        // -------------------------------------------------        InnerRadius
        public int InnerRadius
        {
            get { return (nInRadius); }
            set
            {
                if (nMinInRadius <= value && value <= nOutRadius - nMinWidth)
                {
                    nInRadius = value;
                }
            }
        }
        // -------------------------------------------------        OuterRadius
        public int OuterRadius
        {
            get { return (nOutRadius); }
            set
            {
                if (nInRadius + nMinWidth <= value)
                {
                    nOutRadius = value;
                }
            }
        }
        // -------------------------------------------------        Frame
        public Rectangle Frame
        {
            get { return (rcFrame); }
        }
        // -------------------------------------------------        Values
        public double [] Values
        {
            get { return (fVal); }
            set
            {
                if (value != null && value .Length > 1)
                {
                    int nSector = fVal .Length;
                    int nNewSector = value .Length;

                    fVal = new double [nNewSector];
                    fStart_Deg = new double [nNewSector];
                    fSweep_Deg = new double [nNewSector];
                    for (int i = 0; i < nNewSector; i++)
                    {
                        if (value [i] <= 0)
                        {
                            value [i] = 0.5;
                        }
                        fVal [i] = value [i];
                    }
                    StartAndSweep (ref fStart_Deg, ref fSweep_Deg);

                    if (nSector != nNewSector)
                    {
                        // this will change PART of the arrays
                        Color [] clrs_new = new Color [nNewSector];
                        Color [] clrsText_new = new Color [nNewSector];
                        int nCopyLen = Math .Min (nSector, nNewSector);
                        for (int i = 0; i < nCopyLen; i++)
                        {
                            clrs_new [i] = clrs [i];
                            clrsText_new [i] = clrsText [i];
                        }
                        if (nSector < nNewSector)
                        {
                            for (int i = nCopyLen; i < nNewSector; i++)
                            {
                                clrs_new [i] = Auxi_Colours .ColorPredefined (i);
                                clrsText_new [i] = Color .Black;
                            }
                        }
                        clrs = clrs_new;
                        clrsText = clrsText_new;
                    }
                }
            }
        }
        // -------------------------------------------------        SetValue
        public void SetValue (int i, double val)
        {
            if (0 <= i && i < fVal .Length)
            {
                fVal [i] = Math .Abs (val);
                StartAndSweep (ref fStart_Deg, ref fSweep_Deg);
            }
        }
        // -------------------------------------------------        Colors
        public Color [] Colors
        {
            get { return (clrs); }
            set
            {
                if (clrs != null && clrs .Length >= value .Length)
                {
                    clrs = value;
                }
            }
        }
        // -------------------------------------------------		ColorChange
        public void ColorChange (int i, Color clr)
        {
            if (0 <= i && i < clrs .Length)
            {
                clrs [i] = clr;
            }
        }
        // -------------------------------------------------		ColorMove
        public bool ColorMove (int iFrom, int iTo)
        {
            if (0 <= iFrom && iFrom < clrs .Length && 0 <= iTo && iTo < clrs .Length && iFrom != iTo)
            {
                Color clrMove = clrs [iFrom];
                Color clrTextMove = clrsText [iFrom];
                if (iFrom < iTo)
                {
                    for (int i = iFrom; i < iTo; i++)
                    {
                        clrs [i] = clrs [i + 1];
                        clrsText [i] = clrsText [i + 1];
                    }
                }
                else
                {
                    for (int i = iFrom; i > iTo; i--)
                    {
                        clrs [i] = clrs [i - 1];
                        clrsText [i] = clrsText [i - 1];
                    }
                }
                clrs [iTo] = clrMove;
                clrsText [iTo] = clrTextMove;
                return (true);
            }
            else
            {
                return (false);
            }
        }
        // -------------------------------------------------		TextColor
        public Color TextColor (int i)
        {
            i = Math .Min (Math .Max (0, i), clrsText .Length);
            return (clrsText [i]);
        }
        // -------------------------------------------------		SetTextColor
        public void SetTextColor (int i, Color clr)
        {
            if (0 <= i && i < clrs .Length)
            {
                clrsText [i] = clr;
            }
        }
        // -------------------------------------------------        BorderColor
        public Color BorderColor
        {
            get { return (clrBorder); }
            set { clrBorder = value; }
        }
        // -------------------------------------------------        Rotation
        public Rotation Rotation
        {
            get { return (dirFlow); }
        }
        // -------------------------------------------------        StartingAngle
        public int StartingAngle
        {
            get { return (nStartAngle_Deg); }
            set
            {
                nStartAngle_Deg = Auxi_Common .ChangedAngle (value, 0);
                StartAndSweep (ref fStart_Deg, ref fSweep_Deg);
            }
        }
        // -------------------------------------------------        StartingAngles
        public double [] StartingAngles
        {
            get { return (fStart_Deg); }
        }
        // -------------------------------------------------        SweepingAngles
        public double [] SweepingAngles
        {
            get { return (fSweep_Deg); }
        }
        // -------------------------------------------------        ShowBorders
        public bool ShowBorders
        {
            get { return (bShowBorders); }
            set { bShowBorders = value; }
        }
        // -------------------------------------------------        ShowTexts
        public bool ShowTexts
        {
            get { return (bShowTexts); }
            set { bShowTexts = value; }
        }
        // -------------------------------------------------        ShowValues
        public ShowValue ShowValues
        {
            get { return (showvalue); }
            set { showvalue = value; }
        }
        // -------------------------------------------------        TextsFont
        public Font TextsFont
        {
            get { return (fontTexts); }
            set { fontTexts = value; }
        }
        // -------------------------------------------------        RectAroundDoughnut
        private Rectangle RectAroundDoughnut
        {
            get
            {
                return (new Rectangle (ptCenter .X - nOutRadius, ptCenter .Y - nOutRadius,
                                       2 * nOutRadius, 2 * nOutRadius));
            }
        }
        // -------------------------------------------------        Sectors
        public int Sectors
        {
            get { return (fVal .Length); }
        }

    }
}
