using System;
using System .Collections .Generic;
using System .Drawing;
using System .Windows .Forms;

using MoveGraphLibrary;

namespace Test_MoveGraphLibrary
{
    public partial class Form_ColoredSquares : Form
    {
        int nID;
        Random rand;
        int nSeed;
        decimal decSideMin, decSideMax, decSideIncrement;
        Color clrBack, clrText;
        Font font;

        List<TwoNodeSquare> squares = new List<TwoNodeSquare> ();
        Mover Movers = new Mover ();

        int cyLine;
        string [] strInfo = new string [] { "All colored squares are moveable.", 
                                            "L_Press or R_Press them to move.",
                                            "Select 2- or 4-nodes contours.",
                                            "4-nodes contours cover bigger area." };

        // -------------------------------------------------
        public Form_ColoredSquares ()
        {
            InitializeComponent ();
        }
        // -------------------------------------------------        OnLoad
        private void OnLoad (object sender, EventArgs e)
        {
            TMGL_Common .OneColumnPanelInfo (this, panelInfo, Font, strInfo, out cyLine);

            DateTime dt = DateTime .Now;
            nSeed = dt .Hour * 3600 + dt .Minute * 60 + dt .Second;
            while (nSeed < 10000)
            {
                nSeed += 1000;
            }
            rand = new Random (nSeed);

            decSideMin = 20;
            decSideMax = 120;
            decSideIncrement = 5;
            numericUD_Side .Minimum = decSideMin;
            numericUD_Side .Maximum = decSideMax;
            numericUD_Side .Increment = decSideIncrement;
            RandomCombination ();
            font = new Font ("Times New Roman", 10, FontStyle .Bold | FontStyle .Italic);
            labelSample .Font = font;
            toolTip1 .SetToolTip (btnFont, Auxi_Convert .FontInDetails (font));

            radioTwoNodes .Checked = true;
            int nNodes = radioTwoNodes .Checked ? 2 : 4;
            nID = 0;
            TwoNodeSquare tns = new TwoNodeSquare (new Rectangle (100, 100, 200, 200), 
                                                   Color .Blue, Color .Black, nID .ToString (), Font, nNodes);
            nID++;
            squares .Add (tns);
            Movers .Add (tns);
            Movers .Add (panelInfo);
        }
        // -------------------------------------------------        OnPaint_panelInfo
        private void OnPaint_panelInfo (object sender, PaintEventArgs e)
        {
            Graphics grfx = e .Graphics;
            SolidBrush brush = new SolidBrush (Color .Black);
            for (int i = 0; i < strInfo .Length; i++)
            {
                grfx .DrawString (strInfo [i], Font, brush, 0, i * cyLine);
            }
        }
        // -------------------------------------------------        OnPaint
        private void OnPaint (object sender, PaintEventArgs e)
        {
            Graphics grfx = e .Graphics;
            for (int i = squares .Count - 1; i >= 0; i--)
            {
                TwoNodeSquare tns = squares [i];
                tns .Draw (grfx);
            }
        }
        // -------------------------------------------------        OnMouseDown
        private void OnMouseDown (object sender, MouseEventArgs mea)
        {
            Movers .CatchMover (mea .Location);
        }
        // -------------------------------------------------        OnMouseUp
        private void OnMouseUp (object sender, MouseEventArgs e)
        {
            Movers .ReleaseMover ();
        }
        // -------------------------------------------------        OnMouseMove
        private void OnMouseMove (object sender, MouseEventArgs mea)
        {
            Movers .MovingMover (mea .Location);
            if (Movers .MoverCaught)
            {
                Invalidate ();
            }
        }
        // -------------------------------------------------        OnMouseDoubleClick
        private void OnMouseDoubleClick (object sender, MouseEventArgs mea)
        {
            Point pt = mea .Location;
            foreach (TwoNodeSquare tns in squares)
            {
                if (tns .Inside (pt))
                {
                    if (mea .Button == MouseButtons .Left)
                    {
                        tns .Angle = Auxi_Common .ChangedAngle (tns .Angle, 90);
                        Invalidate ();
                        break;
                    }
                    else if (mea .Button == MouseButtons .Right)
                    {
                        tns .Angle = Auxi_Common .ChangedAngle (tns .Angle, -90);
                        Invalidate ();
                        break;
                    }
                }
            }
        }
        // =================================================        RandomCombination
        private void RandomCombination ()
        {
            int nSideMin = Convert .ToInt32 (numericUD_Side .Minimum);
            int nSideMax = Convert .ToInt32 (numericUD_Side .Maximum);
            int nIncrement = Convert .ToInt32 (numericUD_Side .Increment);

            int nDif = Convert .ToInt32 ((numericUD_Side .Maximum - numericUD_Side .Minimum) / numericUD_Side .Increment);
            numericUD_Side .Value = numericUD_Side .Minimum + numericUD_Side .Increment * (rand .Next (nSeed) % nDif);
            int nSide = Convert .ToInt32 (numericUD_Side .Value);
            labelSample .Text = nSide .ToString ();
            int iClr = rand .Next (nSeed) % nSide;
            int iTextClr = rand .Next (nSeed) % nSide;
            if (iClr == iTextClr) iTextClr ++;
            clrBack = Auxi_Colours .ColorPredefined (iClr);
            clrText = Auxi_Colours .ColorPredefined (iTextClr);
            labelSample .BackColor = clrBack;
            labelSample .ForeColor = clrText;
            toolTip1 .SetToolTip (btnBackClr, Auxi_Convert .ClrToStr (clrBack, ColorStr .IN_ROWS));
            toolTip1 .SetToolTip (btnTextClr, Auxi_Convert .ClrToStr (clrText, ColorStr .IN_ROWS));
        }
        // -------------------------------------------------		Click_BackClr
        private void Click_BackClr (object sender, System .EventArgs ea)
        {
            ColorDialog dlg = new ColorDialog ();

            dlg .Color = clrBack;
            if (dlg .ShowDialog () == DialogResult .OK)
            {
                clrBack = dlg .Color;
                labelSample .BackColor = clrBack;
                toolTip1 .SetToolTip (btnBackClr, Auxi_Convert .ClrToStr (clrBack, ColorStr .IN_ROWS));
            }
        }
        // -------------------------------------------------		Click_TextClr
        private void Click_TextClr (object sender, System .EventArgs ea)
        {
            ColorDialog dlg = new ColorDialog ();

            dlg .Color = clrText;
            if (dlg .ShowDialog () == DialogResult .OK)
            {
                clrText = dlg .Color;
                labelSample .ForeColor = clrText;
                toolTip1 .SetToolTip (btnTextClr, Auxi_Convert .ClrToStr (clrText, ColorStr .IN_ROWS));
            }
        }
        // -------------------------------------------------		Click_Font
        private void Click_Font (object sender, EventArgs e)
        {
            FontDialog dlg = new FontDialog ();

            dlg .Font = font;
            if (dlg .ShowDialog () == DialogResult .OK)
            {
                font = dlg .Font;
                labelSample .Font = font;
                toolTip1 .SetToolTip (btnFont, Auxi_Convert .FontInDetails (font));
            }
        }
        // -------------------------------------------------		Click_btnNewSquare
        private void Click_btnNewSquare (object sender, EventArgs e)
        {
            int nSide = Convert .ToInt32 (numericUD_Side .Value);
            int cxL = panelNew .Right + rand .Next (nSeed) % (ClientSize .Width - (panelNew .Right + nSide));
            int cyT = panelNew .Bottom + rand .Next (nSeed) % (ClientSize .Height - (panelNew .Bottom + nSide));
            int nNodes = radioTwoNodes .Checked ? 2 : 4;
            TwoNodeSquare tns = new TwoNodeSquare (new Rectangle (cxL, cyT, nSide, nSide), clrBack, clrText,
                                                   nID .ToString (), font, nNodes);
            nID++;
            squares .Insert (0, tns);
            Movers .Insert (0, tns);
            Invalidate ();
            RandomCombination ();
        }
    }
}