using System;
using System .Collections .Generic;
using System .Drawing;
using System .Windows .Forms;

using MoveGraphLibrary;

namespace Test_MoveGraphLibrary
{
    public partial class Form_DoughnutSet : Form
    {
        DoughnutSet dntSet;
        int nMaxSectors = 14;
        bool bRingChanged = false;

        Mover Movers = new Mover ();
        bool bShowContours = false;

        int cyLine;
        string [] strInfo = new string [] { "L_Press on connection to move the whole figure.", 
                                            "L_Press on node to change the width of the ring.",
                                            "R_Press on node to rotate the ring." };

        // ------------------------------------------------
        public Form_DoughnutSet ()
        {
            InitializeComponent ();
        }
        // -------------------------------------------------        OnLoad
        private void OnLoad (object sender, EventArgs e)
        {
            TMGL_Common .OneColumnPanelInfo (this, panelInfo, Font, strInfo, out cyLine);

            dntSet = new DoughnutSet (new Point (350, 290), 60, 100, Auxi_Common .RandomArray (170, 5), Font);
            dntSet .AddRing (120, 180, Auxi_Common .RandomArray (500, 10), Font);
            dntSet .AddRing (170, 230, Auxi_Common .RandomArray (90, 3), Font);

            numericUD_Ring .Minimum = 0;
            numericUD_Ring .Maximum = dntSet .RingCount - 1;
            numericUD_Ring .Value = numericUD_Ring .Maximum;

            numericUD_Sectors .Maximum = nMaxSectors;
            btnDelRing .Enabled = dntSet .RingCount > 1;

            RenewMover ();

            toolTip1 .SetToolTip (buttonContours, "Contours ON / OFF");
        }
        // -------------------------------------------------        RenewMover
        private void RenewMover ()
        {
            Movers .Clear ();
            Movers .Add (dntSet);
            Movers .Add (panelRenew);
            Movers .Add (panelInfo);
            Movers .MoverCaught = false;
        }
        // -------------------------------------------------        OnPaint_panelInfo
        private void OnPaint_panelInfo (object sender, PaintEventArgs e)
        {
            Graphics grfx = e .Graphics;
            SolidBrush brush = new SolidBrush (Color .Black);
            for (int i = 0; i < strInfo .Length; i++)
            {
                grfx .DrawString (strInfo [i], Font, brush, 0, i * cyLine);
            }
        }
        // -------------------------------------------------        OnPaint
        private void OnPaint (object sender, PaintEventArgs e)
        {
            Graphics grfx = e .Graphics;
            dntSet .Draw (grfx);
            if (bShowContours)
            {
                for (int i = 0; i < Movers .Count; i++)
                {
                    Movers [i] .DrawContour (grfx);
                }
            }
        }
        // -------------------------------------------------        ValueChanged_numericRing
        private void ValueChanged_numericRing (object sender, EventArgs e)
        {
            decimal dclRing = numericUD_Ring .Value;
            DoughnutRing ring = dntSet .Ring (Convert .ToInt32 (dclRing));
            bRingChanged = true;
            numericUD_Sectors .Value = ring .Sectors;
        }
        // -------------------------------------------------        Click_btnDelRing
        private void Click_btnDelRing (object sender, EventArgs e)
        {
            int curRing = Convert .ToInt32 (numericUD_Ring .Value);
            dntSet .DeleteRing (curRing);
            if (curRing == dntSet .RingCount)
            {
                curRing--;
            }
            numericUD_Ring .Maximum = dntSet .RingCount - 1;
            numericUD_Ring .Value = curRing;
            btnDelRing .Enabled = dntSet .RingCount > 1;
            Invalidate ();
            RenewMover ();
        }
        // -------------------------------------------------        ValueChanged_numericSectors
        private void ValueChanged_numericSectors (object sender, EventArgs e)
        {
            int curRing = Convert .ToInt32 (numericUD_Ring .Value);
            DoughnutRing ring = dntSet .Ring (curRing);
            int nSectors = Convert .ToInt32 (numericUD_Sectors .Value);
            if (!bRingChanged)
            {
                ring .Values = Auxi_Common .RandomArray (250, nSectors);
            }
            bRingChanged = false;
            Invalidate ();
            RenewMover ();
        }
        // -------------------------------------------------        Click_btnRenewRing
        private void Click_btnRenewRing (object sender, EventArgs e)
        {
            int curRing = Convert .ToInt32 (numericUD_Ring .Value);
            DoughnutRing ring = dntSet .Ring (curRing);
            int nSectors = Convert .ToInt32 (numericUD_Sectors .Value);
            ring .Values = Auxi_Common .RandomArray (250, nSectors);
            Invalidate ();
            RenewMover ();
        }
        // -------------------------------------------------        Click_btnAddRing
        private void Click_btnAddRing (object sender, EventArgs e)
        {
            int nNewInside = dntSet .OuterRadius + dntSet .MinimumSpace;
            int nNewOutside = nNewInside + dntSet .MinimumWidth;
            dntSet .AddRing (nNewInside, nNewOutside, Auxi_Common .RandomArray (200, 5), Font);

            numericUD_Ring .Maximum = dntSet .RingCount - 1;
            numericUD_Ring .Value = dntSet .RingCount - 1;
            Invalidate ();
            RenewMover ();
        }
        // -------------------------------------------------        Click_btnContours
        private void Click_btnContours (object sender, EventArgs e)
        {
            bShowContours = !bShowContours;
            Invalidate ();
        }
        // -------------------------------------------------        OnMouseDown
        private void OnMouseDown (object sender, MouseEventArgs mea)
        {
            Movers .CatchMover (mea .Location, mea .Button);
        }
        // -------------------------------------------------        OnMouseUp
        private void OnMouseUp (object sender, MouseEventArgs mea)
        {
            Movers .ReleaseMover ();
        }
        // -------------------------------------------------        OnMouseMove
        private void OnMouseMove (object sender, MouseEventArgs mea)
        {
            Movers .MovingMover (mea .Location);
            if (Movers .MoverCaught)
            {
                Invalidate ();
            }
        }
    }
}