using System;
using System .Collections .Generic;
using System .Drawing;
using System .Windows .Forms;

namespace Test_MoveGraphLibrary
{
    public partial class Form_HouseParams : Form
    {
        public event EventHandler Changed;
        SimpleHouse house;
        Rectangle rcHouse;
        Point ptTop;
        Color clrEdge = Color .DarkGray;
        Rectangle rcDoor, rcWindow;

        // -------------------------------------------------
        public Form_HouseParams (SimpleHouse houseSrc)
        {
            InitializeComponent ();

            house = houseSrc;
            Text = "Paint the house number " + house .Number .ToString ();

            int cxSpace = ClientRectangle .Width / 8;
            int cyLT = ClientRectangle .Height * 5 / 12;    // / 3;
            rcHouse = new Rectangle (cxSpace, cyLT, ClientRectangle .Width - 2 * cxSpace, ClientRectangle .Height / 2);
            ptTop = new Point ((rcHouse .Left + rcHouse .Right) / 2, 10);   
            int roomSize = Math .Min (rcHouse .Height, rcHouse .Width / 2);
            int nW = roomSize / 2;
            int nH = roomSize / 2;
            rcWindow = new Rectangle (rcHouse .Right - roomSize + (roomSize - nW) / 2,
                                      rcHouse .Bottom - roomSize + roomSize / 6, nW, nW);
            nH = roomSize * 2 / 3;
            nW = nH / 2;
            rcDoor = new Rectangle (rcHouse .Left + (roomSize - nW) / 2, rcHouse .Bottom - nH, nW, nH);

            buttonRoof .Location = new Point (ptTop .X - buttonRoof .Width / 2, ptTop .Y + buttonRoof .Height);
            buttonHouse .Location = new Point (rcHouse .Left + 10, rcHouse .Top + 10);
            buttonDoor .Location = new Point (rcDoor .Right - buttonDoor .Width * 2 / 3,
                                              rcDoor .Bottom - buttonDoor .Height * 2 / 3);
            buttonWindow .Location = new Point (rcWindow .Right - buttonWindow .Width * 2 / 3,
                                                rcWindow .Bottom - buttonWindow .Height * 2 / 3);
            btnNumberColor .Location = new Point (ptTop .X + btnNumberColor .Width / 2,
                                                  rcHouse .Top - (btnNumberColor .Height + 1));
            btnNumberFont .Location = new Point (btnNumberColor .Right + 6, btnNumberColor .Top);
        }
        // -------------------------------------------------        OnPaint
        private void OnPaint (object sender, PaintEventArgs e)
        {
            Graphics grfx = e .Graphics;
            grfx .FillRectangle (new SolidBrush (house .HouseColor), rcHouse);
            grfx .DrawRectangle (new Pen (clrEdge), rcHouse);
            Point [] ptRoof = new Point [3] { new Point (rcHouse .Left, rcHouse .Top), ptTop, 
                                              new Point (rcHouse .Right, rcHouse .Top) };
            grfx .FillPolygon (new SolidBrush (house .RoofColor), ptRoof);
            grfx .DrawPolygon (new Pen (clrEdge), ptRoof);
            grfx .FillRectangle (new SolidBrush (house .WindowColor), rcWindow);
            grfx .DrawRectangle (new Pen (clrEdge), rcWindow);
            grfx .FillRectangle (new SolidBrush (house .DoorColor), rcDoor);
            grfx .DrawRectangle (new Pen (clrEdge), rcDoor);
            // Number
            string str = house .Number .ToString ();
            SizeF sizef = grfx .MeasureString (str, house .Font);
            grfx .DrawString (str, house .Font, new SolidBrush (house .NumberColor),
                              ptTop .X - (int) sizef .Width / 2, rcHouse .Top - (int) sizef .Height);
        }
        // -------------------------------------------------        Click_btnRoof
        private void Click_btnRoof (object sender, EventArgs e)
        {
            ColorDialog dlg = new ColorDialog ();

            dlg .Color = house .RoofColor;
            if (dlg .ShowDialog () == DialogResult .OK)
            {
                house .RoofColor = dlg .Color;
                Invalidate ();
                if (Changed != null) Changed (this, new EventArgs ());
            }
        }
        // -------------------------------------------------        Click_btnHouse
        private void Click_btnHouse (object sender, EventArgs e)
        {
            ColorDialog dlg = new ColorDialog ();

            dlg .Color = house .HouseColor;
            if (dlg .ShowDialog () == DialogResult .OK)
            {
                house .HouseColor = dlg .Color;
                Invalidate ();
                if (Changed != null) Changed (this, new EventArgs ());
            }
        }
        // -------------------------------------------------        Click_btnDoor
        private void Click_btnDoor (object sender, EventArgs e)
        {
            ColorDialog dlg = new ColorDialog ();

            dlg .Color = house .DoorColor;
            if (dlg .ShowDialog () == DialogResult .OK)
            {
                house .DoorColor = dlg .Color;
                Invalidate ();
                if (Changed != null) Changed (this, new EventArgs ());
            }
        }
        // -------------------------------------------------        Click_btnWindow
        private void Click_btnWindow (object sender, EventArgs e)
        {
            ColorDialog dlg = new ColorDialog ();

            dlg .Color = house .WindowColor;
            if (dlg .ShowDialog () == DialogResult .OK)
            {
                house .WindowColor = dlg .Color;
                Invalidate ();
                if (Changed != null) Changed (this, new EventArgs ());
            }
        }
        // -------------------------------------------------        Click_btnNumberColor
        private void Click_btnNumberColor (object sender, EventArgs e)
        {
            ColorDialog dlg = new ColorDialog ();

            dlg .Color = house .NumberColor;
            if (dlg .ShowDialog () == DialogResult .OK)
            {
                house .NumberColor = dlg .Color;
                Invalidate ();
                if (Changed != null) Changed (this, new EventArgs ());
            }
        }
        // -------------------------------------------------		Click_btnNumberFont
        private void Click_btnNumberFont (object sender, EventArgs e)
        {
            FontDialog dlg = new FontDialog ();

            dlg .Font = house .Font;
            if (dlg .ShowDialog () == DialogResult .OK)
            {
                house .Font = dlg .Font;
                Invalidate ();
                if (Changed != null) Changed (this, new EventArgs ());
            }
        }
    }
}