using System;
using System .Collections .Generic;
using System .Drawing;
using System .Windows .Forms;
using System .IO;
using Microsoft .Win32;

using MoveGraphLibrary;

namespace Test_MoveGraphLibrary
{
    public partial class Form_Houses : Form
    {
        Mover Movers = new Mover ();
        List<SimpleHouse> Town = new List<SimpleHouse> ();
        int iTouchedHouse;

        // -------------------------------------------------
        public Form_Houses ()
        {
            InitializeComponent ();
        }
        // -------------------------------------------------        OnLoad
        private void OnLoad (object sender, EventArgs e)
        {
            ReadFromRegistry ();
            if (Town .Count == 0)
            {
                SimpleHouse house = new SimpleHouse (1, Font, 40, new Rectangle (150, 150, 100, 60));
                Town .Insert (0, house);
                Movers .Add (house);
            }
        }
        // -------------------------------------------------        OnFormClosing
        private void OnFormClosing (object sender, FormClosingEventArgs e)
        {
            WriteToRegistry ();
        }
        // -------------------------------------------------        OnPaint
        private void OnPaint (object sender, PaintEventArgs e)
        {
            Graphics grfx = e .Graphics;

            for (int i = Town .Count - 1; i >= 0; i --)
            {
                Town [i] .Draw (grfx);
            }
        }
        // -------------------------------------------------		OnMouseDown
        private void OnMouseDown (object sender, MouseEventArgs mea)
        {
            if (mea .Button == MouseButtons .Left)
            {
                Movers .CatchMover (mea .Location);
            }
            else if (mea .Button == MouseButtons .Right)
            {
                for (int i = 0; i < Town .Count; i++)
                {
                    if (Town [i] .Inside (mea .Location))
                    {
                        iTouchedHouse = i;
                        ContextMenuStrip = contextMenuOnHouse;
                        return;
                    }
                }
                ContextMenuStrip = null;
            }
            if (!Movers .MoverCaught)
            {
                BringToTop (mea .Location);
            }
        }
        // -------------------------------------------------		OnMouseUp
        private void OnMouseUp (object sender, MouseEventArgs e)
        {
            Movers .ReleaseMover ();
        }
        // -------------------------------------------------        OnMouseMove
        private void OnMouseMove (object sender, MouseEventArgs mea)
        {
            Cursor cursor = Cursors .Default;

            foreach (SimpleHouse house in Town)
            {
                if (house .Inside (mea .Location))
                {
                    Cursor .Current = Cursors .Hand;
                    break;
                }
            }
            Movers .MovingMover (mea .Location);
            if (Movers .MoverCaught)
            {
                Invalidate ();
            }
        }
        // -------------------------------------------------        OnMouseDoubleClick
        private void OnMouseDoubleClick (object sender, MouseEventArgs mea)
        {
            if (mea .Button == MouseButtons .Left)
            {
                if (Town [0] .Inside (mea .Location))
                {
                    Form_HouseParams form = new Form_HouseParams (Town [0]);
                    form .Changed += new EventHandler (dlgHouseParam_Changed);
                    form .ShowDialog ();
                }
            }
        }
        // -------------------------------------------------		dlgHouseParam_Changed
        private void dlgHouseParam_Changed (object sender, EventArgs ea)
        {
            Invalidate ();
        }
        // =================================================		BringToTop
        private void BringToTop (Point pt)
        {
            for (int i = 1; i < Town .Count; i++)
            {
                SimpleHouse sh = Town [i];

                if (sh .Inside (pt))
                {
                    Town .RemoveAt (i);
                    Town .Insert (0, sh);
                    RenewMover ();
                    Invalidate ();
                    break;
                }
            }
        }
        // =================================================		RenewMover
        private void RenewMover ()
        {
            Movers .Clear ();
            foreach (SimpleHouse house in Town)
            {
                Movers .Add (house);
            }
        }
        // -------------------------------------------------		Click_btnNewHouse
        private void Click_btnNewHouse (object sender, EventArgs e)
        {
            int nNew = 1;
            foreach (SimpleHouse sh in Town)
            {
                nNew = Math .Max (nNew, sh .Number);
            }
            nNew++;
            Button btn = (Button) sender;
            SimpleHouse newhouse = new SimpleHouse (nNew, Font, 16 + (nNew % 10) * 3,
                                                    new Rectangle (btn .Left, btn .Bottom + 20, 100, 60));
            Town .Insert (0, newhouse);
            Movers .Add (newhouse);
            Invalidate ();
        }
        const string strAddRegKey = "Form_Houses";
        const string nameSizes = "ClientArea";
        // =================================================        WriteToRegistry
        private void WriteToRegistry ()
        {
            string strkey = Form_Main .strRegKey + strAddRegKey;
            string nameHouse;
            RegistryKey regkey = null;
            try
            {
                regkey = Registry .CurrentUser .OpenSubKey (strkey, true);
                if (regkey == null)
                {
                    regkey = Registry .CurrentUser .CreateSubKey (strkey);
                }
                string [] strSizes = new string [Town .Count + 3];
                strSizes [0] = ClientSize .Width .ToString ();            
                strSizes [1] = ClientSize .Height .ToString (); 
                strSizes [2] = Town .Count .ToString ();
                for (int i = 0; i < Town .Count; i++)
                {
                    strSizes [i + 3] = Town [i] .Number .ToString ();
                }
                regkey .SetValue (nameSizes, strSizes, RegistryValueKind .MultiString);
                for (int i = 0; i < Town .Count; i++)
                {
                    nameHouse = "House_" + Town [i] .Number .ToString ();
                    string [] strHouse = {  Town [i] .Number .ToString (),      // 0
                                            Town [i] .Area .Left .ToString (),  // 1
                                            Town [i] .Area .Top .ToString (),   // 2
                                            Town [i] .Area .Width .ToString (), // 3
                                            Town [i] .Area .Height .ToString (),    // 4
                                            Town [i] .Top .X .ToString (),      // 5
                                            Town [i] .Top .Y .ToString (),      // 6
                                            Town [i] .RoomSize .ToString (),    // 7
                                            ((int) (Town [i] .HouseColor .R)) .ToString (), // 8
                                            ((int) (Town [i] .HouseColor .G)) .ToString (), // 9
                                            ((int) (Town [i] .HouseColor .B)) .ToString (), // 10
                                            ((int) (Town [i] .RoofColor .R)) .ToString (),  // 11
                                            ((int) (Town [i] .RoofColor .G)) .ToString (),  // 12
                                            ((int) (Town [i] .RoofColor .B)) .ToString (),  // 13
                                            ((int) (Town [i] .WindowColor .R)) .ToString (), // 14
                                            ((int) (Town [i] .WindowColor .G)) .ToString (), // 15
                                            ((int) (Town [i] .WindowColor .B)) .ToString (), // 16
                                            ((int) (Town [i] .DoorColor .R)) .ToString (),  // 17
                                            ((int) (Town [i] .DoorColor .G)) .ToString (),  // 18
                                            ((int) (Town [i] .DoorColor .B)) .ToString (),   // 19
                                            Town [i] .Font .Name .ToString (), // 20
                                            Town [i] .Font .SizeInPoints .ToString (), // 21
                                            ((int) (Town [i] .Font .Style)) .ToString (),  // 22
                                            ((int) (Town [i] .NumberColor .R)) .ToString (),  // 23
                                            ((int) (Town [i] .NumberColor .G)) .ToString (),  // 24
                                            ((int) (Town [i] .NumberColor .B)) .ToString ()   // 25
                                        };
                    regkey .SetValue (nameHouse, strHouse, RegistryValueKind .MultiString);
                }
            }
            catch
            {
            }
            finally
            {
                if (regkey != null) regkey .Close ();
            }
        }
        // =================================================        ReadFromRegistry
        private void ReadFromRegistry ()
        {
            string strkey = Form_Main .strRegKey + strAddRegKey;

            RegistryKey regkey = null;
            try
            {
                regkey = Registry .CurrentUser .OpenSubKey (strkey, true);
                if (regkey != null)
                {
                    int nHouses;
                    string [] strSizes = (string []) regkey .GetValue (nameSizes);
                    if (strSizes != null && strSizes .Length >= 3)
                    {
                        ClientSize = new Size (Convert .ToInt32 (strSizes [0]), Convert .ToInt32 (strSizes [1]));
                        nHouses = Convert .ToInt32 (strSizes [2]);
                        if (nHouses > 0)
                        {
                            int [] nNum = new int [nHouses];
                            for (int i = 0; i < nHouses; i++)
                            {
                                nNum [i] = Convert .ToInt32 (strSizes [i + 3]);
                            }
                            for (int i = 0; i < nHouses; i++)
                            {
                                string nameHouse = "House_" + nNum [i] .ToString ();
                                string [] strHouse = (string []) regkey .GetValue (nameHouse);
                                if (strHouse != null && strHouse .Length >= 26)
                                {
                                    int iNumber = Convert .ToInt32 (strHouse [0]);
                                    Rectangle rc = new Rectangle (Convert .ToInt32 (strHouse [1]),
                                                                  Convert .ToInt32 (strHouse [2]),
                                                                  Convert .ToInt32 (strHouse [3]),
                                                                  Convert .ToInt32 (strHouse [4]));
                                    Point pt = new Point (Convert .ToInt32 (strHouse [5]),
                                                          Convert .ToInt32 (strHouse [6]));
                                    int room = Convert .ToInt32 (strHouse [7]);
                                    Color clrHouse = Color .FromArgb (Convert .ToByte (strHouse [8]),
                                                                       Convert .ToByte (strHouse [9]),
                                                                       Convert .ToByte (strHouse [10]));
                                    Color clrRoof = Color .FromArgb (Convert .ToByte (strHouse [11]),
                                                                       Convert .ToByte (strHouse [12]),
                                                                       Convert .ToByte (strHouse [13]));
                                    Color clrWindow = Color .FromArgb (Convert .ToByte (strHouse [14]),
                                                                       Convert .ToByte (strHouse [15]),
                                                                       Convert .ToByte (strHouse [16]));
                                    Color clrDoor = Color .FromArgb (Convert .ToByte (strHouse [17]),
                                                                       Convert .ToByte (strHouse [18]),
                                                                       Convert .ToByte (strHouse [19]));
                                    Font fnt = new Font (strHouse [20],
                                                         (float) Convert .ToDouble (strHouse [21]),
                                                         (FontStyle) Convert .ToInt32 (strHouse [22]));
                                    Color clrNumber = Color .FromArgb (Convert .ToByte (strHouse [23]),
                                                                       Convert .ToByte (strHouse [24]),
                                                                       Convert .ToByte (strHouse [25]));

                                    SimpleHouse house = new SimpleHouse (iNumber, Font, room, rc, pt, clrHouse, clrRoof);
                                    house .WindowColor = clrWindow;
                                    house .DoorColor = clrDoor;
                                    house .NumberColor = clrNumber;
                                    house .Font = fnt;

                                    Town .Add (house);
                                    Movers .Add (house);
                                }
                            }
                        }
                    }
                }
            }
            catch
            {
            }
            finally
            {
                if (regkey != null) regkey .Close ();
            }
        }
        // -------------------------------------------------		Click_SaveIntoClipboard
        private void Click_SaveIntoClipboard (object sender, EventArgs e)
        {
            Bitmap bitmap = new Bitmap (ClientRectangle .Width, ClientRectangle .Height);
            Graphics grfx = Graphics .FromImage (bitmap);
            grfx .Clear (Color .White);
            for (int i = Town .Count - 1; i >= 0; i--)
            {
                Town [i] .Draw (grfx);
            }

            Clipboard .Clear ();
            Clipboard .SetDataObject (bitmap, true);
            grfx .Dispose ();
        }
        // -------------------------------------------------		Click_miEraseHouse
        private void Click_miEraseHouse (object sender, EventArgs e)
        {
            if (DialogResult .Yes == MessageBox .Show (
                                "Do you want to erase house " + Town [iTouchedHouse] .Number .ToString () + " ?",
                                "Erasing house", MessageBoxButtons .YesNo,
                                MessageBoxIcon .Question, MessageBoxDefaultButton .Button1))
            {
                Town .RemoveAt (iTouchedHouse);
                RenewMover ();
                Invalidate ();
            }
        }
        // -------------------------------------------------		Click_miEraseAll
        private void Click_miEraseAll (object sender, EventArgs e)
        {
            if (DialogResult .Yes == MessageBox .Show ("Do you want to erase all houses?",
                                                       "Erasing all houses", MessageBoxButtons .YesNoCancel,
                                                       MessageBoxIcon .Question, MessageBoxDefaultButton .Button1))
            {
                Town .Clear ();
                RenewMover ();
                Invalidate ();
            }
        }
        // -------------------------------------------------		Click_miSave
        private void Click_miSave (object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog ();
            DialogResult dr = dlg .ShowDialog ();
            if (dr == DialogResult .OK)
            {
                FileStream fs;
                BinaryWriter bw = null;
                try
                {
                    fs = new FileStream (dlg .FileName, FileMode .Create, FileAccess .Write, FileShare .Write);
                    bw = new BinaryWriter (fs);

                    string str = "TOWN";
                    bw .Write (str);
                    bw .Write (Town .Count);
                    for (int i = Town .Count - 1; i >= 0; i--)
                    {
                        bw .Write (Town [i] .Number);
                        Rectangle area = Town [i] .Area;
                        bw .Write (area .Left);
                        bw .Write (area .Top);
                        bw .Write (area .Width);
                        bw .Write (area .Height);
                        Point pt = Town [i] .Top;
                        bw .Write (pt .X);
                        bw .Write (pt .Y);
                        bw .Write (Town [i] .RoomSize);
                        Color clrHouse = Town [i] .HouseColor;
                        bw .Write (clrHouse .R);
                        bw .Write (clrHouse .G);
                        bw .Write (clrHouse .B);
                        Color clrRoof = Town [i] .RoofColor;
                        bw .Write (clrRoof .R);
                        bw .Write (clrRoof .G);
                        bw .Write (clrRoof .B);
                        Color clrWindow = Town [i] .WindowColor;
                        bw .Write (clrWindow .R);
                        bw .Write (clrWindow .G);
                        bw .Write (clrWindow .B);
                        Color clrDoor = Town [i] .DoorColor;
                        bw .Write (clrDoor .R);
                        bw .Write (clrDoor .G);
                        bw .Write (clrDoor .B);

                        Font font = Town [i] .Font;
                        bw .Write (font .Name);
                        bw .Write ((double) (font .SizeInPoints));
                        bw .Write ((int) (font .Style));
                        Color clrNumber = Town [i] .NumberColor;
                        bw .Write (clrNumber .R);
                        bw .Write (clrNumber .G);
                        bw .Write (clrNumber .B);
                    }
                }
                catch
                {
                    string strText = "Can't write into file" + dlg .FileName;
                    MessageBox .Show (strText, "Problem on Saving", MessageBoxButtons .OK, MessageBoxIcon .Information);
                }
                finally
                {
                    bw .Close ();
                }
            }
        }
        // -------------------------------------------------		Click_miRead
        private void Click_miRead (object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog ();
            if (DialogResult .OK == dlg .ShowDialog ())
            {
                FileStream fs;
                BinaryReader br = null;

                try
                {
                    fs = new FileStream (dlg .FileName, FileMode .Open, FileAccess .Read);
                    br = new BinaryReader (fs);

                    string str = br .ReadString ();
                    if (str == "TOWN")
                    {
                        Town .Clear ();
                        Movers .Clear ();
                        int nHouse = br .ReadInt32 ();
                        for (int i = 0; i < nHouse; i++)
                        {
                            int nNum = br .ReadInt32 ();
                            int cxLeft = br .ReadInt32 ();
                            int cyTop = br .ReadInt32 ();
                            int nWidth = br .ReadInt32 ();
                            int nHeight = br .ReadInt32 ();
                            int ptX = br .ReadInt32 ();
                            int ptY = br .ReadInt32 ();
                            int nRoom = br .ReadInt32 ();
                            Color clrHouse  = Color .FromArgb (br .ReadByte (), br .ReadByte (), br .ReadByte ());
                            Color clrRoof   = Color .FromArgb (br .ReadByte (), br .ReadByte (), br .ReadByte ());
                            Color clrWindow = Color .FromArgb (br .ReadByte (), br .ReadByte (), br .ReadByte ());
                            Color clrDoor   = Color .FromArgb (br .ReadByte (), br .ReadByte (), br .ReadByte ());
                            string fntname = br .ReadString ();
                            float fsize = (float) (br .ReadDouble ());
                            FontStyle style = (FontStyle) (br .ReadInt32 ());
                            Color clrNumber = Color .FromArgb (br .ReadByte (), br .ReadByte (), br .ReadByte ());

                            SimpleHouse house = new SimpleHouse (nNum, Font, nRoom,
                                                                 new Rectangle (cxLeft, cyTop, nWidth, nHeight),
                                                                 new Point (ptX, ptY), clrHouse, clrRoof);
                            house .WindowColor = clrWindow;
                            house .DoorColor = clrDoor;
                            house .NumberColor = clrNumber;
                            house .Font = new Font (fntname, fsize, style);

                            Town .Insert (0, house);
                            Movers .Add (house);
                        }
                        Invalidate ();
                    }
                }
                finally
                {
                    br .Close ();
                }
            }
        }
        // -------------------------------------------------		Click_btnHelp
        private void Click_btnHelp (object sender, EventArgs e)
        {
            Form_HouseConstruction form = new Form_HouseConstruction ();
            form .ShowDialog ();
        }

    }
}
