using System;
using System .Collections .Generic;
using System .Drawing;
using System .Windows .Forms;
using Microsoft .Win32;

using MoveGraphLibrary;

namespace Test_MoveGraphLibrary
{
    public partial class Form_Main : Form
    {
        static public string strRegKey = "Software\\Applications_C#\\Test_MoveGraphLibrary\\";
        FullGrArea area;
        DoughnutSet dntSet;
        SimpleHouse houseW, houseF;
        RegularPolygon rp4, rp7;
        Mover Movers;
        bool bShowContours = false;  // needed only if the contours will be shown (for example, by clicking the button) 
        int cyLine;
        string [] strInfo = new string [] { "These are the samples of the objects,", 
                                            "used throughout this application,",
                                            "to demonstrate the design and use of",
                                            "moveable / resizable graphics." };

        // -------------------------------------------------
        public Form_Main ()
        {
            InitializeComponent ();
        }
        // -------------------------------------------------        OnLoad
        private void OnLoad (object sender, EventArgs e)
        {
            TMGL_Common .OneColumnPanelInfo (this, panelInfo, Font, strInfo, out cyLine);

            area = new FullGrArea (new Rectangle (50, 60, 400, 290),
                                   Side .S, 0.0, 4 * Math .PI, Math .PI,
                                   Side .E, 1.0, -1.0, 0.5);
            area .Title .Text = "Sin (x) * Cos (12x)";

            dntSet = new DoughnutSet (new Point (550, 380), 30, 70, Auxi_Common .RandomArray (100, 5), Font);
            dntSet .AddRing (80, 120, Auxi_Common .RandomArray (600, 7), Font);
            dntSet .AddRing (135, 170, Auxi_Common .RandomArray (160, 4), Font);
            dntSet .ShowTitle = false;

            Font fnt = new Font ("Times New Roman", 14, FontStyle.Bold | FontStyle.Italic);
            Rectangle rc = new Rectangle (690, 110, 120, 210);
            houseW = new SimpleHouse (9, Font, 30, rc, new Point ((rc .Left + rc .Right) / 2, rc .Top - 65), 
                                      Color .White, Color .Brown);
            houseW .NumberColor = Color .White;
            houseW .Font = fnt;
            rc = new Rectangle (580, 90, 130, 110);
            houseF = new SimpleHouse (23, Font, 20, rc, new Point ((rc .Left + rc .Right) / 2, rc .Top - 40),
                                      Color .Yellow, Color .Red);
            houseF .Font = fnt;
            houseF .WindowColor = Color .Blue;

            rp4 = new RegularPolygon (0, new Point (100, 440), 50, 4, Color .Violet, Color .Black, Font);
            rp7 = new RegularPolygon (17, new Point (260, 460), 75, 7, Color .Blue, Color .White, Font);
            rp7 .Angle = 20;

            Movers = new Mover ();
            Movers .Add (dntSet);
            Movers .Add (houseW);
            Movers .Add (houseF);
            Movers .Add (rp4);
            Movers .Add (rp7);
            Movers .Add (area);
            Movers .Add (panelInfo);

            toolTip1 .SetToolTip (buttonContours, "Contours ON / OFF");
        }
        // -------------------------------------------------        OnPaint_panelInfo
        private void OnPaint_panelInfo (object sender, PaintEventArgs e)
        {
            Graphics grfx = e .Graphics;
            SolidBrush brush = new SolidBrush (Color .Black);
            for (int i = 0; i < strInfo .Length; i++)
            {
                grfx .DrawString (strInfo [i], Font, brush, 0, i * cyLine);
            }
        }
        // -------------------------------------------------        OnPaint
        private void OnPaint (object sender, PaintEventArgs e)
        {
            Graphics grfx = e .Graphics;

            dntSet .Draw (grfx);
            houseF .Draw (grfx);
            houseW .Draw (grfx);
            rp4 .Draw (grfx);
            rp7 .Draw (grfx);
            area .Draw (grfx);
            area .DrawYofX (grfx, 0, new Delegate_DblOfDbl (SinxCos12x), SegmentLocation .Anywhere);

            // this part only if you need to show contours (switch them ON / OFF)
            if (bShowContours)
            {
                for (int i = 0; i < Movers .Count; i++)
                {
                    Movers [i] .DrawContour (grfx);
                }
            }
        }
        // -------------------------------------------------		OnMouseDown
        private void OnMouseDown (object sender, MouseEventArgs mea)
        {
            Movers .CatchMover (mea .Location, mea .Button);
        }
        // -------------------------------------------------		OnMouseUp
        private void OnMouseUp (object sender, MouseEventArgs e)
        {
            Movers .ReleaseMover ();
        }
        // -------------------------------------------------        OnMouseMove
        private void OnMouseMove (object sender, MouseEventArgs mea)
        {
            Cursor cursor = Cursors .Default;

            if (area .InsideSensitive (mea .Location))
            {
                Cursor .Current = Cursors .Hand;
            }
            Movers .MovingMover (mea .Location);
            if (Movers .MoverCaught)
            {
                Invalidate ();
            }
        }
        // -------------------------------------------------        Click_btnContours
        private void Click_btnContours (object sender, EventArgs e)
        {
            bShowContours = !bShowContours;
            Invalidate ();
        }
        // -------------------------------------------------        OnMouseDoubleClick
        // DoubleClick is used to open and change (through additional dialogs) the parameters of the plot
        private void OnMouseDoubleClick (object sender, MouseEventArgs mea)
        {
            if (mea .Button == MouseButtons .Left)
            {
                if (area .ShowHorScale && area .InsideHorScale (mea .Location))
                {
                    area .HorScale .ParametersDialog (this, dlgHorScaleParam_Changed, null, null);
                }
                else if (area .ShowVerScale && area .InsideVerScale (mea .Location))
                {
                    area .VerScale .ParametersDialog (this, dlgVerScaleParam_Changed, null, null);
                }
                else if (area .InsideMainArea (mea .Location) || area .InsideTitle (mea .Location))
                {
                    area .ParametersDialog (this, dlgAreaParam_Changed, null, null);
                }
            }
        }
        // -------------------------------------------------		dlgAreaParam_Changed
        private void dlgAreaParam_Changed (object sender, EventArgs ea)
        {
            Invalidate ();
        }
        // -------------------------------------------------		dlgHorScaleParam_Changed
        private void dlgHorScaleParam_Changed (object sender, EventArgs ea)
        {
            Invalidate ();
        }
        // -------------------------------------------------		dlgVerScaleParam_Changed
        private void dlgVerScaleParam_Changed (object sender, EventArgs ea)
        {
            Invalidate ();
        }
        // =================================================        SinxCos12x
        private double SinxCos12x (double fx)
        {
            return (Math .Sin (fx) * Math .Cos (12 * fx));
        }
        // -------------------------------------------------        Click_miUnmovable
        private void Click_miUnmovable (object sender, EventArgs e)
        {
            Form_UnmovablePlots form = new Form_UnmovablePlots ();
            form .ShowDialog ();
        }
        // -------------------------------------------------        Click_miOneMovablePlot
        private void Click_miOneMovablePlot (object sender, EventArgs e)
        {
            Form_OneMovablePlot form = new Form_OneMovablePlot ();
            form .ShowDialog ();
        }
        // -------------------------------------------------        Click_miSeveralPlots
        private void Click_miSeveralPlots (object sender, EventArgs e)
        {
            Form_MovablePlots form = new Form_MovablePlots ();
            form .ShowDialog ();
        }
        // -------------------------------------------------        Click_miHouses
        private void Click_miHouses (object sender, EventArgs e)
        {
            Form_Houses form = new Form_Houses ();
            form .ShowDialog ();
        }
        // -------------------------------------------------        Click_miColoredSquares
        private void Click_miColoredSquares (object sender, EventArgs e)
        {
            Form_ColoredSquares form = new Form_ColoredSquares ();
            form .ShowDialog ();
        }
        // -------------------------------------------------        Click_miRegularPolygons
        private void Click_miRegularPolygons (object sender, EventArgs e)
        {
            Form_RegularPolygons form = new Form_RegularPolygons ();
            form .ShowDialog ();
        }
        // -------------------------------------------------        Click_miRegularPolygons
        private void Click_miDoughnutSet (object sender, EventArgs e)
        {
            Form_DoughnutSet form = new Form_DoughnutSet ();
            form .ShowDialog ();
        }
        // -------------------------------------------------        Click_miAbout
        private void Click_miAbout (object sender, EventArgs e)
        {
            Form_About form = new Form_About ();
            form .ShowDialog ();
        }
    }
}