using System;
using System .Collections .Generic;
using System .Drawing;
using System .Windows .Forms;
using Microsoft .Win32;

using MoveGraphLibrary;

namespace Test_MoveGraphLibrary
{
    public partial class Form_MovablePlots : Form
    {
        int iTouchedPlot = -1;
        Point ptMouseRClick;
        Mover Movers = new Mover ();
        bool bShowContours = false;
        List<PlotOnScreen> plotInView = new List<PlotOnScreen> ();
        string [] Cmnts = new string [] {"R_Click on empty place ",   "- select new plot", 
                                         "R_Click on existing plot ", "- open context menu", 
                                         "L_DblClick on plot ",       "- change plot's parameters",
                                         "L_DblClick on scale ",      "- change scale's parameters" };
        int cyShiftComments, cxShiftComments_R;

        // -------------------------------------------------
        public Form_MovablePlots ()
        {
            InitializeComponent ();

            SetPanelCommentsSize ();
            panelComments .BackColor = Color .LightYellow;
        }
        // -------------------------------------------------        OnLoad
        private void OnLoad (object sender, EventArgs e)
        {
            plotInView .Clear ();
            RestoreFromRegistry ();

            Movers .Add (panelComments);
            Movers .Add (panelWithList, ContourResize .Any, 325, 400, 106, 320);
            Movers .MoverCaught = false;
            RefreshList ();

            toolTip1 .SetToolTip (buttonContours, "Contours ON / OFF");
        }
        // -------------------------------------------------        OnFormClosing
        private void OnFormClosing (object sender, FormClosingEventArgs e)
        {
            WriteIntoRegistry ();
        }
        // -------------------------------------------------        OnPaint
        private void OnPaint (object sender, PaintEventArgs e)
        {
            Graphics grfx = e .Graphics;

            for (int i = plotInView .Count - 1; i >= 0; i--)
            {
                plotInView [i] .GrArea .Draw (grfx);
                DrawStandardFunction (plotInView [i] .GrArea, plotInView [i] .Type,
                                      grfx, 0, SegmentLocation .Partly_Inside);
                if (bShowContours)
                {
                    Movers [i] .DrawContour (grfx);
                }
            }
            // frame around controls, included into Mover
            if (bShowContours)
            {
                for (int i = 0; i < Movers .Count; i++)
                {
                    if (Movers [i] .Source == Source .Control)
                    {
                        Movers [i] .DrawContour (grfx);
                    }
                }
            }
        }
        // -------------------------------------------------        OnMouseDown
        private void OnMouseDown (object sender, MouseEventArgs mea)
        {
            if (mea .Button == MouseButtons .Left)
            {
                Movers .CatchMover (mea .Location);
                if (!Movers .MoverCaught)
                {
                    // if on plot, Popup it
                    int iPopup = -1;
                    for (int i = 0; i < plotInView .Count; i++)
                    {
                        if (plotInView [i] .GrArea .InsideMainArea (mea .Location))
                        {
                            iPopup = i;
                            if (iPopup > 0)
                            {
                                do
                                {
                                    Movers .Reverse (iPopup - 1, 2);
                                    plotInView .Reverse (iPopup - 1, 2);
                                    iPopup--;
                                } while (iPopup > 0);
                                Invalidate ();
                                RefreshList ();
                                listFunctions .Items [0] .Selected = true;
                                listFunctions .Items [0] .EnsureVisible ();
                            }
                            break;
                        }
                    }
                }
            }
            else if (mea .Button == MouseButtons .Right)
            {
                // on any Plot open ContextMenu, outside any plot - open dialog to choose the function
                iTouchedPlot = -1;
                for (int i = 0; i < plotInView .Count; i++)
                {
                    if (plotInView [i] .GrArea .InsideMainArea (mea .Location))
                    {
                        iTouchedPlot = i;
                        break;
                    }
                }
                if (iTouchedPlot == -1)
                {
                    ContextMenuStrip = null;
                    // EMPTY space,  NOT on any function
                    ptMouseRClick = mea .Location;
                    Point ptScreen = this .PointToScreen (mea .Location);
                    Form_SelectNewFunction form = new Form_SelectNewFunction (this, ptScreen); 
                    form .ShowDialog ();
                    int iNew = form .SelectedFunction;
                    if (iNew >= 0)
                    {
                        int nSelectFunc = Enum .GetNames (typeof (DemoFuncType)) .Length - 1;
                        Rectangle rc = new Rectangle (mea .Location .X, mea .Location .Y,
                                                      ClientSize .Width / 2, ClientSize .Height / 2);
                        if (iNew < nSelectFunc)
                        {
                            // one of standard functions
                            double [] fParam;
                            DemoFuncType functype = (DemoFuncType) iNew;
                            fParam = PlotOnScreen .PredefinedFunctonParams (functype);
                            FullGrArea area = new FullGrArea (rc,
                                                    Side .S, fParam [0], fParam [1], fParam [2],
                                                    Side .E, fParam [3], fParam [4], fParam [5]);
                            AddDemoFunction ((DemoFuncType) iNew, area);  
                            RefreshList ();
                            Invalidate ();
                        }
                    }
                }
                else
                {
                    // ON PLOT
                    ptMouseRClick = mea .Location;
                    ContextMenuStrip = contextMenuOnFunc;
                }
            }
        }
        // -------------------------------------------------        OnMouseUp
        private void OnMouseUp (object sender, MouseEventArgs e)
        {
            Movers .ReleaseMover ();
        }
        // -------------------------------------------------        OnMouseMove
        private void OnMouseMove (object sender, MouseEventArgs e)
        {
            Cursor cursor = Cursors .Default;

            for (int i = plotInView .Count - 1; i >= 0; i--)
            {
                if (plotInView [i] .GrArea .InsideSensitive (e .Location))
                {
                    Cursor .Current = Cursors .Hand;
                    break;
                }
            }
            Movers .MovingMover (e .Location);
            if (Movers .MoverCaught)
            {
                Invalidate ();
            }
        }
        // -------------------------------------------------        OnMouseDoubleClick
        private void OnMouseDoubleClick (object sender, MouseEventArgs mea)
        {
            if (mea .Button == MouseButtons .Left)
            {
                for (int i = 0; i < plotInView .Count; i++)
                {
                    FullGrArea area = plotInView [i] .GrArea;
                    if (area .InsideHorScale (mea .Location) && area .ShowHorScale)
                    {
                        area .HorScale .ParametersDialog (this, dlgHorScaleParam_Changed, null,
                                                          area .Title .Text + " - HOR scale parameters");
                        break;
                    }
                    else if (area .InsideVerScale (mea .Location) && area .ShowVerScale)
                    {
                        area .VerScale .ParametersDialog (this, dlgVerScaleParam_Changed, null,
                                                          area .Title .Text + " - VER scale parameters");
                        break;
                    }
                    else if (area .InsideMainArea (mea .Location))
                    {
                        area .ParametersDialog (this, dlgAreaParam_Changed, null,
                                                area .Title .Text + " - main parameters");
                        break;
                    }
                }
            }
        }
        // -------------------------------------------------		dlgAreaParam_Changed
        private void dlgAreaParam_Changed (object sender, EventArgs ea)
        {
            Invalidate ();
        }
        // -------------------------------------------------		dlgHorScaleParam_Changed
        private void dlgHorScaleParam_Changed (object sender, EventArgs ea)
        {
            Invalidate ();
        }
        // -------------------------------------------------		dlgVerScaleParam_Changed
        private void dlgVerScaleParam_Changed (object sender, EventArgs ea)
        {
            Invalidate ();
        }
        // -------------------------------------------------        Click_btnMoveAround
        private void Click_btnMoveAround (object sender, EventArgs e)
        {
            SwitchFrames ();
        }
        // -------------------------------------------------        SwitchFrames
        private void SwitchFrames ()
        {
            bShowContours = !bShowContours;
            Invalidate ();
        }
        // -------------------------------------------------        Click_miSwitchFrames
        private void Click_miSwitchFrames (object sender, EventArgs e)
        {
            SwitchFrames ();
        }
        // -------------------------------------------------        Click_miGraphAreaParams
        private void Click_miGraphAreaParams (object sender, EventArgs e)
        {
            FullGrArea area = plotInView [iTouchedPlot] .GrArea;
            area .ParametersDialog (this, dlgAreaParam_Changed, null,
                                    area .Title .Text + " - main parameters");
        }
        // -------------------------------------------------        Click_miHorScaleParams
        private void Click_miHorScaleParams (object sender, EventArgs e)
        {
            FullGrArea area = plotInView [iTouchedPlot] .GrArea;
            area .HorScale .ParametersDialog (this, dlgHorScaleParam_Changed, null,
                                              area .Title .Text + " - HOR scale parameters");
        }
        // -------------------------------------------------        Click_miVerScaleParams
        private void Click_miVerScaleParams (object sender, EventArgs e)
        {
            FullGrArea area = plotInView [iTouchedPlot] .GrArea;
            area .VerScale .ParametersDialog (this, dlgVerScaleParam_Changed, null,
                                              area .Title .Text + " - VER scale parameters");
        }
        // -------------------------------------------------        Click_miPurePlotIntoClipboard
        private void Click_miPurePlotIntoClipboard (object sender, EventArgs e)
        {
            PlotIntoClipboard (CopyPart .PurePlot);
        }
        // -------------------------------------------------        Click_miFullGraphAreaIntoClipboard
        private void Click_miFullGraphAreaIntoClipboard (object sender, EventArgs e)
        {
            PlotIntoClipboard (CopyPart .FullPlotArea);
        }
        // =================================================        PlotIntoClipboard
        private void PlotIntoClipboard (CopyPart copy)
        {
            Bitmap bitmapFull = new Bitmap (ClientRectangle .Width, ClientRectangle .Height);
            Graphics gFull = Graphics .FromImage (bitmapFull);
            gFull .Clear (Color .White);

            Rectangle rc = Rectangle .Empty;
            if (copy == CopyPart .PurePlot)
            {
                DrawStandardFunction (plotInView [iTouchedPlot] .GrArea, plotInView [iTouchedPlot] .Type,
                                      gFull, 0, SegmentLocation .Partly_Inside);
                rc = plotInView [iTouchedPlot] .GrArea .MainArea .Area;
            } else if (copy == CopyPart .FullPlotArea) {
                plotInView [iTouchedPlot] .GrArea .Draw (gFull);
                DrawStandardFunction (plotInView [iTouchedPlot] .GrArea, plotInView [iTouchedPlot] .Type,
                                      gFull, 0, SegmentLocation .Partly_Inside);
                rc = plotInView [iTouchedPlot] .GrArea .Frame;
            }
            Bitmap bitmapPart = new Bitmap (rc .Width, rc .Height);
            Graphics gPart = Graphics .FromImage (bitmapPart);
            gPart .Clear (Color .White);
            gPart .DrawImage (bitmapFull, 0, 0, rc, GraphicsUnit .Pixel);
            gPart .Dispose ();
            gFull .Dispose ();

            Clipboard .Clear ();
            Clipboard .SetDataObject (bitmapPart, true);
        }
        // -------------------------------------------------        Click_miClientAreaIntoClipboard
        private void Click_miClientAreaIntoClipboard (object sender, EventArgs e)
        {
            Bitmap bitmap = new Bitmap (ClientRectangle .Width, ClientRectangle .Height);
            Graphics grfx = Graphics .FromImage (bitmap);
            grfx .Clear (Color .White);

            for (int i = plotInView .Count - 1; i >= 0; i--)
            {
                plotInView [i] .GrArea .Draw (grfx);
                DrawStandardFunction (plotInView [i] .GrArea, plotInView [i] .Type,
                                      grfx, 0, SegmentLocation .Partly_Inside);
            }
            Clipboard .Clear ();
            Clipboard .SetDataObject (bitmap, true);
            grfx .Dispose ();
        }
        // -------------------------------------------------        Click_miFullPlotWhiteBack
        private void Click_miFullPlotWhiteBack (object sender, EventArgs e)
        {
            Bitmap bitmapFull = new Bitmap (ClientRectangle .Width, ClientRectangle .Height);
            Graphics gFull = Graphics .FromImage (bitmapFull);
            gFull .Clear (Color .White);

            Rectangle rc = Rectangle .Empty;

            FullGrArea areaTmp = FullGrArea .Copy (plotInView [iTouchedPlot] .GrArea);
            areaTmp .MainArea .BackColor = Color .White;
            areaTmp .Draw (gFull);
            DrawStandardFunction (plotInView [iTouchedPlot] .GrArea, plotInView [iTouchedPlot] .Type,
                                  gFull, 0, SegmentLocation .Partly_Inside);

            rc = plotInView [iTouchedPlot] .GrArea .Frame;

            Bitmap bitmapPart = new Bitmap (rc .Width, rc .Height);
            Graphics gPart = Graphics .FromImage (bitmapPart);
            gPart .Clear (Color .White);
            gPart .DrawImage (bitmapFull, 0, 0, rc, GraphicsUnit .Pixel);
            gPart .Dispose ();
            gFull .Dispose ();

            Clipboard .Clear ();
            Clipboard .SetDataObject (bitmapPart, true);
        }
        // -------------------------------------------------        Click_miDeleteTouchedArea
        private void Click_miDeleteTouchedArea (object sender, EventArgs e)
        {
            plotInView [iTouchedPlot] .GrArea .CloseTuningForms ();
            Movers .RemoveAt (iTouchedPlot);
            plotInView .RemoveAt (iTouchedPlot);
            Invalidate ();
            RefreshList ();
        }

        // =================================================        AddDemoFunction
        // new function is added on top of the list
        private void AddDemoFunction (DemoFuncType type, FullGrArea area)
        {
            int iNew = 0;
            PlotOnScreen demoFunc = new PlotOnScreen (type, area);
            plotInView .Insert (0, demoFunc);
            RefreshList ();
            MovableObject mover = new MovableObject (plotInView [iNew] .GrArea);
            Movers .Insert (iNew, mover);
        }
        // =================================================        RefreshList
        private void RefreshList ()
        {
            listFunctions .Items .Clear ();
            for (int i = 0; i < plotInView .Count; i++)
            {
                string [] arstr = new string [4];
                arstr [0] = i .ToString ();
                arstr [1] = plotInView [i] .GrArea .Title .Text;
                NumScale scale = plotInView [i] .GrArea .HorScale;
                string [] strEnds = scale .EndValuesInDefinedFormat;
                arstr [2] = strEnds [0];
                arstr [3] = strEnds [1];
                ListViewItem lvi = new ListViewItem (arstr);
                listFunctions .Items .Add (lvi);
            }
        }
        // =================================================		DrawStandardFunction
        private static void DrawStandardFunction (FullGrArea area, DemoFuncType type, Graphics grfx, Pen pen,
                                                  SegmentLocation nSegmentToShow)
        {
            double [] fParam;

            switch (type)
            {
                case DemoFuncType .Sine:
                    area .DrawYofX (grfx, pen, new Delegate_DblOfDbl (Math .Sin), SegmentLocation .Partly_Inside); 
                    break;
                case DemoFuncType .Cosine:
                    area .DrawYofX (grfx, pen, new Delegate_DblOfDbl (Math .Cos), SegmentLocation .Partly_Inside); 
                    break;
                case DemoFuncType .HiperbolicTangent:
                    area .DrawYofX (grfx, pen, new Delegate_DblOfDbl (Math .Tanh), SegmentLocation .Partly_Inside); 
                    break;
                case DemoFuncType .Arcsine:
                    area .DrawYofX (grfx, pen, new Delegate_DblOfDbl (Math .Asin), SegmentLocation .Partly_Inside); 
                    break;
                case DemoFuncType .Arccosine:
                    area .DrawYofX (grfx, pen, new Delegate_DblOfDbl (Math .Acos), SegmentLocation .Partly_Inside); 
                    break;
                case DemoFuncType .Arctangent:
                    area .DrawYofX (grfx, pen, new Delegate_DblOfDbl (Math .Atan), SegmentLocation .Partly_Inside); 
                    break;
                case DemoFuncType .SinSin:
                    fParam = new double [2] { 14, 1 };
                    area .DrawYofX (grfx, pen, new Delegate_DblOfArray (Sample_Func .sinAsinB), fParam,
                                    SegmentLocation .Partly_Inside);  
                    break;
                case DemoFuncType .DampedVibrations:
                    fParam = new double [] { 3, 2, 10 };
                    area .DrawYofX (grfx, pen, new Delegate_DblOfArray (Sample_Func .RepeatDampedVibrations), fParam,
                                    SegmentLocation .Partly_Inside); 
                    break;
                case DemoFuncType .Distribution:
                    fParam = new double [2] { 0.8, 0.5 };
                    area .DrawYofX (grfx, pen, new Delegate_DblOfArray (Sample_Func .NormalDistribution), fParam,
                                    SegmentLocation .Partly_Inside); 
                    break;
                case DemoFuncType .Parametric_0:
                    area .DrawParamFunc (grfx, pen, new Delegate_DblOfDbl (Sample_Func .ParamFunc_X_0),
                                                    new Delegate_DblOfDbl (Sample_Func .ParamFunc_Y_0),
                                                    0.0, 29.0, 0.02);  
                    break;
                case DemoFuncType .Parametric_1:
                    area .DrawParamFunc (grfx, pen, new Delegate_DblOfDbl (Sample_Func .ParamFunc_X_1),
                                                    new Delegate_DblOfDbl (Sample_Func .ParamFunc_Y_1),
                                                    -7.5, 7.5, 0.02);  
                    break;
                case DemoFuncType .Parametric_2:
                    area .DrawParamFunc (grfx, pen, new Delegate_DblOfDbl (Sample_Func .ParamFunc_X_2),
                                                    new Delegate_DblOfDbl (Sample_Func .ParamFunc_Y_2),
                                                    -4.0, 4.0, 0.01);  
                    break;
                case DemoFuncType .Lissajou:
                    area .DrawParamFunc (grfx, pen, new Delegate_DblOfDbl (Sample_Func .Lissajou_Param_X),
                                                    new Delegate_DblOfDbl (Sample_Func .Lissajou_Param_Y),
                                                    0.0, 20.0, 0.02);  
                    break;
                case DemoFuncType .Parametric_4:
                    area .DrawParamFunc (grfx, pen, new Delegate_DblOfDbl (Sample_Func .ParamFunc_X_4),
                                                    new Delegate_DblOfDbl (Sample_Func .ParamFunc_Y_4),
                                                    -2.5, 2.5, 0.005); 
                    break;
                case DemoFuncType .Parametric_5:
                    area .DrawParamFunc (grfx, pen, new Delegate_DblOfDbl (Sample_Func .ParamFunc_X_5),
                                                    new Delegate_DblOfDbl (Sample_Func .ParamFunc_Y_5),
                                                    0.0, 6.29, 0.01);
                    break;
                case DemoFuncType .Trigonometric_0:
                    area .DrawYofX (grfx, pen, new Delegate_DblOfDbl (Math .Sin), SegmentLocation .Partly_Inside);
                    area .DrawYofX (grfx, pen, new Delegate_DblOfDbl (Math .Cos), SegmentLocation .Partly_Inside);
                    break;
            }
        }
        // =================================================		DrawStandardFunction
        private static void DrawStandardFunction (FullGrArea area, DemoFuncType type, Graphics grfx, int iPen,
                                                 SegmentLocation nSegmentToShow)  
        {
            Pen [] pens = area .MainArea .Pens;
            iPen = Math .Min (Math .Max (0, iPen), pens .Length - 1);
            if (type < DemoFuncType .Trigonometric_0)
            {
                DrawStandardFunction (area, type, grfx, pens [iPen], nSegmentToShow);
            }
            else
            {
                area .DrawYofX (grfx, pens [0], new Delegate_DblOfDbl (Math .Sin), SegmentLocation .Partly_Inside);
                area .DrawYofX (grfx, pens [1], new Delegate_DblOfDbl (Math .Cos), SegmentLocation .Partly_Inside);
            }
        }
        // -------------------------------------------------        Click_btnUp
        private void Click_btnUp (object sender, EventArgs e)
        {
            if (listFunctions .SelectedIndices .Count > 0)
            {
                int iSel = listFunctions .SelectedIndices [0];
                if (0 < iSel && iSel < plotInView .Count)
                {                       
                    Movers .Reverse (iSel - 1, 2);
                    plotInView .Reverse (iSel - 1, 2);
                    Invalidate ();
                    RefreshList ();
                    listFunctions .Items [iSel - 1] .Selected = true;
                    listFunctions .Items [iSel - 1] .EnsureVisible ();
                }
            }
        }
        // -------------------------------------------------        Click_btnDown
        private void Click_btnDown (object sender, EventArgs e)
        {
            if (listFunctions .SelectedIndices .Count > 0)
            {
                int iSel = listFunctions .SelectedIndices [0];
                if (0 <= iSel && iSel < plotInView .Count - 1)
                {                      
                    Movers .Reverse (iSel, 2);
                    plotInView .Reverse (iSel, 2);
                    Invalidate ();
                    RefreshList ();
                    listFunctions .Items [iSel + 1] .Selected = true;
                    listFunctions .Items [iSel + 1] .EnsureVisible ();
                }
            }
        }
        // -------------------------------------------------        OnPaint_panelComments
        private void OnPaint_panelComments (object sender, PaintEventArgs e)
        {
            Graphics grfx = e .Graphics;
            SolidBrush brush = new SolidBrush (Color .Black);
            int nLines = Cmnts .Length / 2;
            for (int i = 0; i < nLines; i++)
            {
                grfx .DrawString (Cmnts [i * 2], Font, brush, 0, i * cyShiftComments);
                grfx .DrawString (Cmnts [i * 2 + 1], Font, brush, cxShiftComments_R, i * cyShiftComments);
            }
        }
        // =================================================        SetPanelCommentsSize
        private void SetPanelCommentsSize ()
        {
            Graphics grfx = this .CreateGraphics ();
            int nW_L = 0;
            int nW_R = 0;
            int nH = 0;
            int nLines = Cmnts .Length / 2;
            for (int i = 0; i < nLines; i++)
            {
                string str_L = Cmnts [i * 2];
                string str_R = Cmnts [i * 2 + 1];
                SizeF sizef_L = grfx .MeasureString (str_L, Font);
                SizeF sizef_R = grfx .MeasureString (str_R, Font);
                nW_L = Math .Max (nW_L, Convert .ToInt32 (sizef_L .Width));
                nW_R = Math .Max (nW_R, Convert .ToInt32 (sizef_R .Width));
                cyShiftComments = Convert .ToInt32 (sizef_L .Height);
                nH += cyShiftComments;
            }
            grfx .Dispose ();
            cxShiftComments_R = nW_L;
            panelComments .Width = nW_L + nW_R + 4;
            panelComments .Height = nH + 6;
        }

        const string strAddRegKey = "Form_SeveralPlots";
        const string nameFuncTypes = "FuncTypes";
        const string nameCommon = "Common";
        // =================================================        WriteIntoRegistry
        private void WriteIntoRegistry ()
        {
            string strRegKey = Form_Main .strRegKey + strAddRegKey;

            RegistryKey regkey = null;
            try
            {
                regkey = Registry .CurrentUser .OpenSubKey (strRegKey, true);
                if (regkey == null)
                {
                    regkey = Registry .CurrentUser .CreateSubKey (strRegKey);
                }
                int nFunc = plotInView .Count;
                string [] strSizes = {  ClientSize .Width .ToString (),         // 0            
                                        ClientSize .Height .ToString (),         // 1 
                                        panelWithList .Bounds .X .ToString (),         // 2
                                        panelWithList .Bounds .Y .ToString (),         // 3
                                        panelComments .Bounds .X .ToString (),         // 4
                                        panelComments .Bounds .Y .ToString (),         // 5 
                                        nFunc .ToString (),          // 6
                                        panelWithList .Width .ToString (),         // 7
                                        panelWithList .Height .ToString (),         // 8 
                                        listFunctions .Width .ToString (),         // 9
                                        listFunctions .Height .ToString (),         // 10
                                        listFunctions .Columns [0] .Width .ToString (),     // 11
                                        listFunctions .Columns [1] .Width .ToString (),     // 12
                                        listFunctions .Columns [2] .Width .ToString (),     // 13
                                        listFunctions .Columns [3] .Width .ToString ()      // 14
                                        //buttonUp .Bounds .X .ToString (),         // 11
                                        //buttonUp .Bounds .Y .ToString (),         // 12
                                        //buttonDown .Bounds .Y .ToString ()         // 13
                                      };
                regkey .SetValue (nameCommon, strSizes, RegistryValueKind .MultiString);

                string [] strTypes = new string [nFunc];
                for (int i = 0; i < nFunc; i++)
                {
                    strTypes [i] = ((int) (plotInView [i] .Type)) .ToString ();
                }
                regkey .SetValue (nameFuncTypes, strTypes, RegistryValueKind .MultiString);

                for (int i = 0; i < nFunc; i++)
                {
                    char ch = (char) ('A' + i);
                    string str = ch .ToString ();
                    plotInView [i] .GrArea .IntoRegistry (regkey, str);
                }
            }
            catch
            {
            }
            finally
            {
                if (regkey != null) regkey .Close ();
            }
        }
        // =================================================        RestoreFromRegistry
        private void RestoreFromRegistry ()
        {
            string namekey = Form_Main .strRegKey + strAddRegKey;

            RegistryKey regkey = null;
            try
            {
                regkey = Registry .CurrentUser .OpenSubKey (namekey, true);
                if (regkey != null)
                {
                    int nFunc = 0;
                    string [] strSizes = (string []) regkey .GetValue (nameCommon);
                    if (strSizes != null  &&  strSizes .Length >= 15)
                    {
                        ClientSize = new Size (Convert .ToInt32 (strSizes [0]), Convert .ToInt32 (strSizes [1]));
                        panelWithList .Location = new Point (Convert .ToInt32 (strSizes [2]),
                                                             Convert .ToInt32 (strSizes [3]));
                        panelComments .Location = new Point (Convert .ToInt32 (strSizes [4]),
                                                             Convert .ToInt32 (strSizes [5]));
                        nFunc = Convert .ToInt32 (strSizes [6]);
                        panelWithList .Width = Convert .ToInt32 (strSizes [7]);
                        panelWithList .Height = Convert .ToInt32 (strSizes [8]);
                        listFunctions .Width = Convert .ToInt32 (strSizes [9]);
                        listFunctions .Height = Convert .ToInt32 (strSizes [10]);
                        listFunctions .Columns [0] .Width = Convert .ToInt32 (strSizes [11]);
                        listFunctions .Columns [1] .Width = Convert .ToInt32 (strSizes [12]);
                        listFunctions .Columns [2] .Width = Convert .ToInt32 (strSizes [13]);
                        listFunctions .Columns [3] .Width = Convert .ToInt32 (strSizes [14]);
                        //buttonUp .Location = new Point (Convert .ToInt32 (strSizes [11]),
                        //                                Convert .ToInt32 (strSizes [12]));
                        //buttonDown .Location = new Point (Convert .ToInt32 (strSizes [11]),
                        //                                  Convert .ToInt32 (strSizes [13]));
                    }
                    if (nFunc > 0)
                    {
                        // types of functions
                        int [] nTypes = new int [nFunc];
                        string [] strTypes = (string []) regkey .GetValue (nameFuncTypes);
                        for (int i = 0; i < nFunc; i++)
                        {
                            nTypes [i] = Convert .ToInt32 (strTypes [i]);
                        }
                        // areas
                        FullGrArea [] area = new FullGrArea [nFunc];
                        for (int i = 0; i < nFunc; i++)
                        {
                            char ch = (char) ('A' + i);
                            string str = ch .ToString ();
                            area [i] = new FullGrArea (regkey, str);
                        }
                        for (int i = nFunc - 1; i >= 0; i--)
                        {
                            AddDemoFunction ((DemoFuncType) nTypes [i], area [i]);
                        }
                    }
                }
            }
            catch
            {
            }
            finally
            {
                if (regkey != null) regkey .Close ();
            }
        }

        // -------------------------------------------------        ClientSizeChanged_panelWithList
        // this was used to keep buttons in the middle of the vertical size
        // with the new location and anchoring of buttons this method is not needed at all
        private void ClientSizeChanged_panelWithList (object sender, EventArgs e)
        {
            /*
            Rectangle rcUp = buttonUp .Bounds;
            Rectangle rcDown = buttonDown .Bounds;
            int space = rcDown .Top - rcUp .Bottom;
            int cyMiddle = ((Panel) sender) .ClientRectangle .Height / 2;
            buttonUp .Top = cyMiddle - (space / 2 + buttonUp .Height);
            buttonDown .Top = cyMiddle + space / 2;
            */
        }

    }
}