using System;
using System .Collections .Generic;
using System .Drawing;
using System .Windows .Forms;
using Microsoft .Win32;

using MoveGraphLibrary;

namespace Test_MoveGraphLibrary
{
    public partial class Form_OneMovablePlot : Form
    {
        FullGrArea area;
        Mover Movers;
        bool bShowContours = false;
        const string strAddRegKey = "Form_OneMovable";
        const string nameSizes = "ClientArea";

        // -------------------------------------------------
        public Form_OneMovablePlot ()
        {
            InitializeComponent ();
        }
        // -------------------------------------------------        OnLoad
        private void OnLoad (object sender, EventArgs e)
        {
            area = new FullGrArea (new Rectangle (50, 60, 560, 400),
                                   Side .S, 0.0, 4 * Math .PI, Math .PI,
                                   Side .E, 1.0, -1.0, 0.5);
            area .Title .Text = "Sin (x) * Cos (20x)";
            ReadFromRegistry ();

            Movers = new Mover ();
            Movers .Add (area);

            toolTip1 .SetToolTip (buttonContours, "Contours ON / OFF");
        }
        // -------------------------------------------------        OnFormClosing
        private void OnFormClosing (object sender, FormClosingEventArgs e)
        {
            WriteToRegistry ();
        }
        // -------------------------------------------------        OnPaint
        private void OnPaint (object sender, PaintEventArgs e)
        {
            Graphics grfx = e .Graphics;

            area .Draw (grfx);
            area .DrawYofX (grfx, 0, new Delegate_DblOfDbl (SinxCos20x), SegmentLocation .Anywhere);

            // this part only if you need to show contours (switch them ON / OFF)
            if (bShowContours)
            {
                for (int i = 0; i < Movers .Count; i++)
                {
                    Movers [i] .DrawContour (grfx);
                }
            }
        }
        // -------------------------------------------------		OnMouseDown
        private void OnMouseDown (object sender, MouseEventArgs mea)
        {
            if (mea .Button == MouseButtons .Left)
            {
                Movers .CatchMover (mea .Location);
            }
        }
        // -------------------------------------------------		OnMouseUp
        private void OnMouseUp (object sender, MouseEventArgs e)
        {
            Movers .ReleaseMover ();
        }
        // -------------------------------------------------        OnMouseMove
        private void OnMouseMove (object sender, MouseEventArgs mea)
        {
            Cursor cursor = Cursors .Default;

            if (area .InsideSensitive (mea .Location))
            {
                Cursor .Current = Cursors .Hand;
            }
            Movers .MovingMover (mea .Location);
            if (Movers .MoverCaught)
            {
                Invalidate ();
            }
        }
        // -------------------------------------------------        OnMouseDoubleClick
        // DoubleClick is used to open and change (through additional dialogs) the parameters of the plot
        private void OnMouseDoubleClick (object sender, MouseEventArgs mea)
        {
            if (mea .Button == MouseButtons .Left)
            {
                if (area .ShowHorScale && area .InsideHorScale (mea .Location))
                {
                    area .HorScale .ParametersDialog (this, dlgHorScaleParam_Changed, dlgHorScaleParam_Closed, null);
                }
                else if (area .ShowVerScale && area .InsideVerScale (mea .Location))
                {
                    area .VerScale .ParametersDialog (this, dlgVerScaleParam_Changed, dlgVerScaleParam_Closed, null);
                }
                else if (area .InsideMainArea (mea .Location) || area .InsideTitle (mea .Location))
                {
                    area .ParametersDialog (this, dlgAreaParam_Changed, dlgAreaParam_Closed, null);
                }
            }
        }
        // -------------------------------------------------		dlgAreaParam_Changed
        private void dlgAreaParam_Changed (object sender, EventArgs ea)
        {
            Invalidate ();
        }
        // -------------------------------------------------		dlgHorScaleParam_Changed
        private void dlgHorScaleParam_Changed (object sender, EventArgs ea)
        {
            Invalidate ();
        }
        // -------------------------------------------------		dlgVerScaleParam_Changed
        private void dlgVerScaleParam_Changed (object sender, EventArgs ea)
        {
            Invalidate ();
        }
        // -------------------------------------------------		dlgAreaParam_Closed
        private void dlgAreaParam_Closed (object sender, EventArgs ea)
        {
        }
        // -------------------------------------------------		dlgHorScaleParam_Closed
        private void dlgHorScaleParam_Closed (object sender, EventArgs ea)
        {
        }
        // -------------------------------------------------		dlgVerScaleParam_Closed
        private void dlgVerScaleParam_Closed (object sender, EventArgs ea)
        {
        }
        // -------------------------------------------------        Click_btnContours
        private void Click_btnContours (object sender, EventArgs e)
        {
            bShowContours = !bShowContours;
            Invalidate ();
        }

        // =================================================        SinxCos12x
        private double SinxCos20x (double fx)
        {
            return (Math .Sin (fx) * Math .Cos (20 * fx));
        }
        // =================================================        ReadFromRegistry
        private void ReadFromRegistry ()
        {
            string strkey = Form_Main .strRegKey + strAddRegKey;

            RegistryKey regkey = null;
            try
            {
                regkey = Registry .CurrentUser .OpenSubKey (strkey, true);
                if (regkey != null)
                {
                    string [] strSizes = (string []) regkey .GetValue (nameSizes);
                    if (strSizes != null && strSizes .Length >= 2)
                    {
                        ClientSize = new Size (Convert .ToInt32 (strSizes [0]), Convert .ToInt32 (strSizes [1]));
                    }
                    area = new FullGrArea (regkey, "A");
                }
            }
            catch
            {
            }
            finally
            {
                if (regkey != null) regkey .Close ();
            }
        }
        // =================================================        WriteToRegistry
        private void WriteToRegistry ()
        {
            string strkey = Form_Main .strRegKey + strAddRegKey;

            RegistryKey regkey = null;
            try
            {
                regkey = Registry .CurrentUser .OpenSubKey (strkey, true);
                if (regkey == null)
                {
                    regkey = Registry .CurrentUser .CreateSubKey (strkey);
                }
                string [] strSizes = {  ClientSize .Width .ToString (),            
                                        ClientSize .Height .ToString () };
                regkey .SetValue (nameSizes, strSizes, RegistryValueKind .MultiString);
                area .IntoRegistry (regkey, "A");
            }
            catch
            {
            }
            finally
            {
                if (regkey != null) regkey .Close ();
            }
        }
    }
}