using System;
using System .Drawing;
using System .Windows .Forms;
using MoveGraphLibrary;

namespace Test_MoveGraphLibrary
{
    public partial class Form_UnmovablePlots : Form
    {
        FullGrArea area_0, area_1;

        Mover Movers = new Mover ();
        int cyLine;
        string [] strInfo = new string [] { "Only this panel is moveable here.", 
                                            "Plots are neither moveable, nor resizable,",
                                            "but L_DoubleClick will open the tuning forms." };

        // -------------------------------------------------
        public Form_UnmovablePlots ()
        {
            InitializeComponent ();
        }
        // -------------------------------------------------        OnLoad
        private void OnLoad (object sender, EventArgs e)
        {
            TMGL_Common .OneColumnPanelInfo (this, panelInfo, Font, strInfo, out cyLine);
            Movers .Add (panelInfo);

            area_0 = new FullGrArea (new Rectangle (50, 30, 350, 180),
                                     Side .S, -4, 25, 5,
                                     Side .W, 1.2, -1.0, 0.5);
            // changing the standard view
            area_0 .Title .Text = "Standard sin(x)";
            area_0 .AnchorTitleXcoef = 0.8;
            area_0 .AnchorTitleYcoef = 0.06;
            area_0 .HorScale .DecPlaces = 0;
            area_0 .HorScale .SetRotationCenter (TextBasicPoint .N, 0, 2);

            area_1 = new FullGrArea (new Rectangle (100, 250, 700, 340),
                                     Side .S, 0.0, 18, Math .PI,
                                     Side .E, 1.1, -1.1, 0.5);
            area_1 .Title .Color = Color .Red;
            area_1 .Title .Font = new Font ("Times New Roman", 15);
            area_1 .Title .Text = "Three functions";
            area_1 .AnchorTitleXcoef = 0.8;
            area_1 .AnchorTitleYcoef = -0.05;
            area_1 .VerScale .TextsAngle = -10;
            area_1 .HorScale .ShowNumRB = false;
        }
        // -------------------------------------------------        OnPaint_panelInfo
        private void OnPaint_panelInfo (object sender, PaintEventArgs e)
        {
            Graphics grfx = e .Graphics;
            SolidBrush brush = new SolidBrush (Color .Black);
            for (int i = 0; i < strInfo .Length; i++)
            {
                grfx .DrawString (strInfo [i], Font, brush, 0, i * cyLine);
            }
        }
        // -------------------------------------------------        OnPaint
        private void OnPaint (object sender, PaintEventArgs e)
        {
            Graphics grfx = e .Graphics;

            area_0 .Draw (grfx);
            area_0 .DrawYofX (grfx, new Pen (Color .Blue), new Delegate_DblOfDbl (Math .Sin), SegmentLocation .Anywhere);

            area_1 .Draw (grfx);
            area_1 .DrawYofX (grfx, 0, new Delegate_DblOfDbl (Sin10xCosx), SegmentLocation .Anywhere);
            area_1 .DrawYofX (grfx, new Pen (Color .Red), new Delegate_DblOfDbl (Math .Cos), SegmentLocation .Anywhere);
            area_1 .DrawYofX (grfx, new Pen (Color .Red), new Delegate_DblOfDbl (MinusCosx), SegmentLocation .Anywhere);
        }
        // -------------------------------------------------        Sin5xCosx
        private double Sin10xCosx (double x)
        {
            return (Math .Sin (10 * x) * Math .Cos (x));
        }
        // -------------------------------------------------        MinusCosx
        private double MinusCosx (double x)
        {
            return (- Math .Cos (x));
        }
        // -------------------------------------------------        OnMouseDown
        private void OnMouseDown (object sender, MouseEventArgs mea)
        {
            Movers .CatchMover (mea .Location);
        }
        // -------------------------------------------------        OnMouseUp
        private void OnMouseUp (object sender, MouseEventArgs e)
        {
            Movers .ReleaseMover ();
        }
        // -------------------------------------------------        OnMouseMove
        private void OnMouseMove (object sender, MouseEventArgs mea)
        {
            Cursor cursor = Cursors .Default;

            if (area_0 .InsideSensitive (mea .Location) || area_1 .InsideSensitive (mea .Location))
            {
                Cursor .Current = Cursors .Hand;
            }
            Movers .MovingMover (mea .Location);
            if (Movers .MoverCaught)
            {
                Invalidate ();
            }
        }
        // -------------------------------------------------        OnMouseDoubleClick
        // DoubleClick is used to open and change (through additional dialogs) the parameters of the plots
        private void OnMouseDoubleClick (object sender, MouseEventArgs mea)
        {
            if (mea .Button == MouseButtons .Left)
            {
                if (area_0 .ShowHorScale && area_0 .InsideHorScale (mea .Location))
                {
                    area_0 .HorScale .ParametersDialog (this, ParametersChanged, null, null);
                }
                else if (area_0 .ShowVerScale && area_0 .InsideVerScale (mea .Location))
                {
                    area_0 .VerScale .ParametersDialog (this, ParametersChanged, null, null);
                }
                else if (area_0 .InsideMainArea (mea .Location) || area_0 .InsideTitle (mea .Location))
                {
                    area_0 .ParametersDialog (this, ParametersChanged, null, null);
                }
                else if (area_1 .ShowHorScale && area_1 .InsideHorScale (mea .Location))
                {
                    area_1 .HorScale .ParametersDialog (this, ParametersChanged, null, null);
                }
                else if (area_1 .ShowVerScale && area_1 .InsideVerScale (mea .Location))
                {
                    area_1 .VerScale .ParametersDialog (this, ParametersChanged, null, null);
                }
                else if (area_1 .InsideMainArea (mea .Location) || area_1 .InsideTitle (mea .Location))
                {
                    area_1 .ParametersDialog (this, ParametersChanged, null, null);
                }
            }
        }
        // -------------------------------------------------		ParametersChanged
        private void ParametersChanged (object sender, EventArgs ea)
        {
            Invalidate ();
        }
    }
}