using System;
using System .Collections .Generic;
using System .Drawing;
using System .Windows .Forms;

using MoveGraphLibrary;

namespace Test_MoveGraphLibrary
{
    class RegularPolygon : GraphicalObject
    {
        int nID;
        Point ptC;
        int nRadius, nRadiusOut;
        int nApexes;
        Color clrBack, clrText;
        int nAngle;
        Font font;

        // -------------------------------------------------        RegularPolygon
        public RegularPolygon (int id, Point center, int radius, int apexes, Color clr, Color clrStr, Font fnt)
        {
            nID = id;
            ptC = center;
            nRadius = Math .Max (Math .Abs (radius), 30);
            nApexes = apexes;
            clrBack = clr;
            clrText = clrStr;
            nAngle = 0;
            font = fnt;
            nRadiusOut = Convert .ToInt32 (nRadius / Math .Cos (Math .PI / nApexes));
        }
        // -------------------------------------------------        Draw
        public void Draw (Graphics grfx)
        {
            double fInitRadian = nAngle * Math .PI / 180;
            Point [] pt = new Point [nApexes];
            for (int i = 0; i < nApexes; i++)
            {
                pt [i] = new Point (ptC .X + (int) (nRadiusOut * Math .Cos (fInitRadian + 2 * Math .PI * i / nApexes)),
                                    ptC .Y + (int) (nRadiusOut * Math .Sin (fInitRadian + 2 * Math .PI * i / nApexes)));
            }
            grfx .FillPolygon (new SolidBrush (clrBack), pt);
            Auxi_Drawing .DrawEllipse (grfx, Color .DarkGray, ptC, nRadius);
            string str = nID .ToString () + " (" + nAngle .ToString () + ")";
            Auxi_Drawing .RotatedText (grfx, str, font, nAngle, clrText, ptC, TextBasicPoint .M);
        }
        // -------------------------------------------------        Apexes
        public int Apexes
        {
            get { return (nApexes); }
        }
        // -------------------------------------------------        Font
        public Font Font
        {
            get { return (font); }
            set { font = value; }
        }
        // -------------------------------------------------        Angle
        public int Angle
        {
            get { return (nAngle); }
            set { nAngle = value; }
        }
        // -------------------------------------------------        Inside
        public bool Inside (Point pt)
        {
            double fInitRadian = nAngle * Math .PI / 180;
            Point [] poly = new Point [nApexes];
            for (int i = 0; i < nApexes; i++)
            {
                poly [i] = new Point (ptC .X + (int) (nRadiusOut * Math .Cos (fInitRadian + 2 * Math .PI * i / nApexes)),
                                      ptC .Y + (int) (nRadiusOut * Math .Sin (fInitRadian + 2 * Math .PI * i / nApexes)));
            }
            return (Auxi_Geometry .InsideConvexPolygon (pt, poly));
        }
        // -------------------------------------------------        DefineContour
        public override void DefineContour ()
        {
            ContourApex [] ca = new ContourApex [1];
            ca [0] = new ContourApex (0, ptC, new Size (0, 0), MovementFreedom .Any, Cursors .Hand);
            ca [0] .SenseAreaSize = nRadius;
            ca [0] .SenseAreaForm = ContourApexForm .Circle;
            ca [0] .SenseAreaClearance = false;
            contour = new Contour (ca, null);
        }
        // -------------------------------------------------
        public override void Move (int cx, int cy)
        {
            ptC += new Size (cx, cy);
        }
        // -------------------------------------------------        MoveContourPoint
        public override bool MoveContourPoint (int i, int cx, int cy, Point ptM, MouseButtons catcher)
        {
            bool bRet = false;
            if (catcher == MouseButtons .Left)
            {
                Move (cx, cy);
                bRet = true;
            }
            return (bRet);
        }

    }
}
