using System;
using System .Collections .Generic;
using System .Drawing;
using System .Windows .Forms;
using MoveGraphLibrary;

namespace Test_MoveGraphLibrary
{
    public class SimpleHouse : GraphicalObject
    {
        int nNumber;
        Font font;
        Rectangle rcHouse;
        Point ptTop;
        int roomSize;
        Color clrHouse = Color .FromArgb (255, 128, 0);
        Color clrRoof = Color .Red;
        Color clrWindow = Color .LightBlue;
        Color clrDoor = Color .White;
        Color clrEdge = Color .DarkGray;
        Color clrNumber = Color .Black;

        int minRoomSize = 16;
        int minWidth;
        int minHeight;
        int minRoofH;
        int minRoofSide;

        int doorH;  // = 12;
        int doorW;  // = 6;
        int windowSize; // = 10;

        // -------------------------------------------------
        public SimpleHouse (int number, Font fnt, int room, Rectangle rc, Point pt, Color houseClr, Color roofClr)
        {
            nNumber = number;
            font = fnt;
            SetSizes (room);
            rcHouse = new Rectangle (rc .Left, rc .Top, Math .Max (minWidth, rc .Width), Math .Max (minHeight, rc .Height));
            ptTop = new Point (Math .Min (Math .Max (rcHouse .Left + roomSize / 2, pt .X), rcHouse .Right - roomSize / 2),
                               Math .Min (pt .Y, rc .Top - minRoofH));
            clrHouse = houseClr;
            clrRoof = roofClr;
        }
        public SimpleHouse (int number, Font fnt, int room, Rectangle rc)
        {
            nNumber = number;
            font = fnt;
            SetSizes (room);
            rcHouse = new Rectangle (rc .Left, rc .Top, Math .Max (minWidth, rc .Width), Math .Max (minHeight, rc .Height));
            ptTop = new Point ((rcHouse .Left + rcHouse .Right) / 2,
                                rcHouse .Top - Math .Max (minRoofH, rcHouse .Height / 4));
        }
        // -------------------------------------------------        SetSizes
        public void SetSizes (int room)
        {
            roomSize = Math .Max (room, minRoomSize);
            minWidth = roomSize * 2;
            minHeight = roomSize * 3 / 2;
            minRoofH = minRoomSize;
            minRoofSide = minRoomSize / 2;
            doorH = roomSize * 2 / 3;
            doorW = doorH / 2;
            windowSize = doorH * 5 / 6;
        }
        // -------------------------------------------------        Draw
        public void Draw (Graphics grfx)
        {
            grfx .FillRectangle (new SolidBrush (clrHouse), rcHouse);
            grfx .DrawRectangle (new Pen (clrEdge), rcHouse);
            Point [] ptRoof = new Point [3] { new Point (rcHouse .Left, rcHouse .Top), ptTop, 
                                              new Point (rcHouse .Right, rcHouse .Top) };
            grfx .FillPolygon (new SolidBrush (clrRoof), ptRoof);
            grfx .DrawPolygon (new Pen (clrEdge), ptRoof);
            // fill with windows
            int nInRow = rcHouse .Width / roomSize;
            int nFloor = rcHouse .Height / roomSize;
            int jDoor = nInRow / 2;
            int cxL = rcHouse .Left + (rcHouse .Width - nInRow * roomSize) / 2;
            for (int i = 0; i < nFloor; i++)
            {
                for (int j = 0; j < nInRow; j++)
                {
                    Rectangle rcRoom = new Rectangle (cxL + j * roomSize, rcHouse .Bottom - (i + 1) * roomSize,
                                                      roomSize, roomSize);
                    if (i == 0 && j == jDoor)
                    {
                        Rectangle rcDoor = new Rectangle (rcRoom .Left + (rcRoom .Width - doorW) / 2, rcRoom .Bottom - doorH,
                                                          doorW, doorH);
                        grfx .FillRectangle (new SolidBrush (clrDoor), rcDoor);
                        grfx .DrawRectangle (new Pen (clrEdge), rcDoor);
                    }
                    else
                    {
                        Rectangle rcWin = new Rectangle (rcRoom .Left + (rcRoom .Width - windowSize) / 2, rcRoom .Top + 4,
                                                          windowSize, windowSize);
                        grfx .FillRectangle (new SolidBrush (clrWindow), rcWin);
                        grfx .DrawRectangle (new Pen (clrEdge), rcWin);
                    }
                }
            }
            // Number
            string str = nNumber .ToString ();
            SizeF sizef = grfx .MeasureString (str, font);
            grfx .DrawString (str, font, new SolidBrush (clrNumber),
                              ptTop .X - (int) sizef .Width / 2, rcHouse .Top - (int) sizef .Height);
        }
        // -------------------------------------------------        Area
        public Rectangle Area
        {
            get { return (rcHouse); }
        }
        // -------------------------------------------------        Top
        public Point Top
        {
            get { return (ptTop); }
        }
        // -------------------------------------------------        RoomSize
        public int RoomSize
        {
            get { return (roomSize); }
        }
        // -------------------------------------------------        HouseColor
        public Color HouseColor
        {
            get { return (clrHouse); }
            set { clrHouse = value; }
        }
        // -------------------------------------------------        RoofColor
        public Color RoofColor
        {
            get { return (clrRoof); }
            set { clrRoof = value; }
        }
        // -------------------------------------------------        DoorColor
        public Color DoorColor
        {
            get { return (clrDoor); }
            set { clrDoor = value; }
        }
        // -------------------------------------------------        WindowColor
        public Color WindowColor
        {
            get { return (clrWindow); }
            set { clrWindow = value; }
        }
        // -------------------------------------------------        NumberColor
        public Color NumberColor
        {
            get { return (clrNumber); }
            set { clrNumber = value; }
        }
        // -------------------------------------------------        Number
        public int Number
        {
            get { return (nNumber); }
        }
        // -------------------------------------------------        Font
        public Font Font
        {
            get { return (font); }
            set { font = value; }
        }
        // -------------------------------------------------        Inside
        public bool Inside (Point pt)
        {
            Point [] pts = new Point [5] { new Point (rcHouse .Left, rcHouse .Top), ptTop, 
                                 new Point (rcHouse .Right, rcHouse .Top), new Point (rcHouse .Right, rcHouse .Bottom), 
                                 new Point (rcHouse .Left, rcHouse .Bottom) };
            return (Auxi_Geometry .InsideConvexPolygon (pt, pts));
        }
        // -------------------------------------------------        DefineContour
        public override void DefineContour ()
        {
            ContourApex [] ca = new ContourApex [5];
            ca [0] = new ContourApex (0, new Point (rcHouse .Left, rcHouse .Top), new Size (0, 0), MovementFreedom .Any, Cursors .Hand);
            ca [1] = new ContourApex (1, new Point (rcHouse .Right, rcHouse .Top), new Size (0, 0), MovementFreedom .Any, Cursors .Hand);
            ca [2] = new ContourApex (2, new Point (rcHouse .Right, rcHouse .Bottom), new Size (0, 0), MovementFreedom .Any, Cursors .Hand);
            ca [3] = new ContourApex (3, new Point (rcHouse .Left, rcHouse .Bottom), new Size (0, 0), MovementFreedom .Any, Cursors .Hand);
            ca [4] = new ContourApex (4, ptTop, new Size (0, 0), MovementFreedom .Any, Cursors .Hand);
            ContourConnection [] cc = new ContourConnection [6] { new ContourConnection (0, 1), new ContourConnection (1, 2),
                                                                  new ContourConnection (2, 3), new ContourConnection (3, 0),
                                                                  new ContourConnection (0, 4), new ContourConnection (1, 4) };
            contour = new Contour (ca, cc);
        }
        // -------------------------------------------------
        // this will move the whole system for (cx, cy) pixels
        public override void Move (int cx, int cy)
        {
            rcHouse .X += cx;
            rcHouse .Y += cy;
            ptTop += new Size (cx, cy);
        }
        // -------------------------------------------------        MoveContourPoint
        // this move depends on individual restrictions
        public override bool MoveContourPoint (int i, int cx, int cy, Point ptM, MouseButtons catcher)
        {
            bool bRet = false;

            if (catcher == MouseButtons .Left)
            {
                if (i == 0)     // LT
                {
                    if (rcHouse .Height - cy >= minHeight)
                    {
                        rcHouse .Y += cy;
                        rcHouse .Height -= cy;
                        ptTop .Y += cy;
                        bRet = true;
                    }
                    if (rcHouse .Width - cx >= minWidth)
                    {
                        rcHouse .X += cx;
                        rcHouse .Width -= cx;
                        ptTop .X += cx;
                        ptTop .X = Math .Min (Math .Max (rcHouse .Left + minRoofSide, ptTop .X), rcHouse .Right - minRoofSide);
                        bRet = true;
                    }
                }
                else if (i == 1)     // RT
                {
                    if (rcHouse .Height - cy >= minHeight)
                    {
                        rcHouse .Y += cy;
                        rcHouse .Height -= cy;
                        ptTop .Y += cy;
                        bRet = true;
                    }
                    if (rcHouse .Width + cx >= minWidth)
                    {
                        rcHouse .Width += cx;
                        ptTop .X += cx;
                        ptTop .X = Math .Min (Math .Max (rcHouse .Left + minRoofSide, ptTop .X), rcHouse .Right - minRoofSide);
                        bRet = true;
                    }
                }
                else if (i == 2)     // RB
                {
                    if (rcHouse .Height + cy >= minHeight)
                    {
                        rcHouse .Height += cy;
                        bRet = true;
                    }
                    if (rcHouse .Width + cx >= minWidth)
                    {
                        rcHouse .Width += cx;
                        ptTop .X += cx;
                        ptTop .X = Math .Min (Math .Max (rcHouse .Left + minRoofSide, ptTop .X), rcHouse .Right - minRoofSide);
                        bRet = true;
                    }
                }
                else if (i == 3)     // LB
                {
                    if (rcHouse .Height + cy >= minHeight)
                    {
                        rcHouse .Height += cy;
                        bRet = true;
                    }
                    if (rcHouse .Width - cx >= minWidth)
                    {
                        rcHouse .X += cx;
                        rcHouse .Width -= cx;
                        ptTop .X += cx;
                        ptTop .X = Math .Min (Math .Max (rcHouse .Left + minRoofSide, ptTop .X), rcHouse .Right - minRoofSide);
                        bRet = true;
                    }
                }
                else if (i == 4)     // Roof top
                {
                    if (ptTop .Y + cy <= rcHouse .Top - minRoofH)
                    {
                        ptTop .Y += cy;
                        bRet = true;
                    }
                    if (rcHouse .Left + minRoofSide <= ptTop .X + cx && ptTop .X + cx <= rcHouse .Right - minRoofSide)
                    {
                        ptTop .X += cx;
                        bRet = true;
                    }
                }
            }
            return (bRet);
        }

    }
}
