using System;
using System .Collections .Generic;
using System .Drawing;
using System .Windows .Forms;

using MoveGraphLibrary;

namespace Test_MoveGraphLibrary
{
    class TwoNodeSquare : GraphicalObject
    {
        Rectangle rc;
        Color clrBack, clrText, clrMark;
        string str;
        int angle;
        int nodes;
        Font font;

        // -------------------------------------------------        TwoNodeSquare
        public TwoNodeSquare (Rectangle rcInit, Color clr, Color clrStr, string text, Font fnt, int nNodes)
        {
            rc = rcInit;
            clrBack = clr;
            clrText = clrStr;
            str = text;
            angle = 0;
            font = fnt; // new Font ("Times New Roman", 12, FontStyle .Bold | FontStyle .Italic); 
            nodes = nNodes;
            clrMark = Color .Black;
        }
        // -------------------------------------------------        Draw
        public void Draw (Graphics grfx)
        {
            grfx .FillRectangle (new SolidBrush (clrBack), rc);
            Auxi_Drawing .RotatedText (grfx, str, font, angle, Color .Black, Auxi_Geometry .Middle (rc), TextBasicPoint .M);
            Auxi_Drawing .FillEllipse (grfx, clrMark, new Point (rc .Left + 5, rc .Top + 5), 3);
            Auxi_Drawing .FillEllipse (grfx, clrMark, new Point (rc .Right - 5, rc .Top + 5), 3);
            if (nodes == 4)
            {
                Auxi_Drawing .FillEllipse (grfx, clrMark, new Point (rc .Left + 5, rc .Bottom - 5), 3);
                Auxi_Drawing .FillEllipse (grfx, clrMark, new Point (rc .Right - 5, rc .Bottom - 5), 3);
            }
        }
        // -------------------------------------------------        Angle
        public int Angle
        {
            get { return (angle); }
            set { angle = value; }
        }
        // -------------------------------------------------        Inside
        public bool Inside (Point pt)
        {
            return (rc .Contains (pt));
        }
        // -------------------------------------------------        DefineContour
        public override void DefineContour ()
        {
            int nToSide;
            ContourApex [] apexes;
            if (nodes == 2)
            {
                nToSide = Math .Min (rc .Width, rc .Height) / 2;
                Point ptM = Auxi_Geometry .Middle (rc);
                apexes = new ContourApex [2] { 
                    new ContourApex (0, new Point (ptM .X - 1, ptM .Y), new Size (0, 0), MovementFreedom.None, Cursors.SizeAll), 
                    new ContourApex (1, new Point (ptM .X + 1, ptM .Y), new Size (0, 0), MovementFreedom.None, Cursors.SizeAll) 
                };
                contour = new Contour (apexes);
                contour .LineSensitivity = nToSide - 1;
            }
            else
            {
                nToSide = Math .Min (rc .Width, rc .Height) / 4;
                int cxL = rc .Left;
                int cxR = rc .Right;
                int cyT = rc .Top;
                int cyB = rc .Bottom;
                apexes = new ContourApex [4] { 
                    new ContourApex (0, new Point (cxL + nToSide, cyT + nToSide), new Size (0, 0), MovementFreedom.None, Cursors.SizeAll), 
                    new ContourApex (1, new Point (cxR - nToSide, cyT + nToSide), new Size (0, 0), MovementFreedom.None, Cursors.SizeAll), 
                    new ContourApex (2, new Point (cxR - nToSide, cyB - nToSide), new Size (0, 0), MovementFreedom.None, Cursors.SizeAll), 
                    new ContourApex (3, new Point (cxL + nToSide, cyB - nToSide), new Size (0, 0), MovementFreedom.None, Cursors.SizeAll) 
                };
                contour = new Contour (apexes);
                contour .LineSensitivity = nToSide;
            }
        }
        // -------------------------------------------------
        public override void Move (int cx, int cy)
        {
            rc .X += cx;
            rc .Y += cy;
        }
        // -------------------------------------------------        MoveContourPoint
        public override bool MoveContourPoint (int i, int cx, int cy, Point ptM, MouseButtons catcher)
        {
            return (false);
        }
    }
}
